/*
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 *  Copyright(C) 2006, NXP BV, All rights reserved.
 *  by Jean-Paul Saman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "stbcfg.h"
#include <common.h>
#include <asm/io.h>
#include <config.h>

/* defined in cpu/arm1176/cpu.c */
extern unsigned long  ChipID, ChipRevID;

#define BOARD_IRD		1

/**************************************************************************
 * get_cpu_type() - low level get cpu type
 * - no C globals yet.
 * - just looking to say if this is a 2422 or 2420 or ...
 * - to start with we will look at switch settings..
 * - 2422 id's same as 2420 for ES1 will rely on H4 board characteristics
 *   (mux for 2420, non-mux for 2422).
 ***************************************************************************/
u32 get_cpu_type(void)
{
	return(ChipID);
}

/******************************************
 * get_cpu_rev(void) - extract version info
 ******************************************/
u32 get_cpu_rev(void)
{
	return(ChipRevID);
}

/***********************************************************
 * get_mem_type() - identify type of mDDR part used.
 *************************************************************/
u32 get_mem_type(void)
{
   return 1;
}

/***********************************************************************
 * get_board_type() - get board type based on current production stats.
 ************************************************************************/
u32 get_board_type(void)
{
	return(BOARD_IRD);
}

/*********************************************************************
 * wait_on_value() - common routine to allow waiting for changes in
 *   volatile regs.
 *********************************************************************/
u32 wait_on_value(u32 read_bit_mask, u32 match_value, u32 read_addr, u32 bound)
{
	u32 i = 0, val;
	do {
		++i;
		val = __raw_readl(read_addr) & read_bit_mask;
		if (val == match_value)
			return(1);
		if (i==bound)
			return(0);
	} while (1);
}

/*********************************************************************
 *  display_board_info() - print banner with board info.
 *********************************************************************/
static char* map_revid_string( u32 revid)
{
   switch( revid )
   { 
      case 0x10:   return "Rev B";
      case 0x20:   return "Rev C";
      case 0x30:   return "Rev D";
      default:     return "NOT SUPPORTED REVISION";
   }
}
void display_board_info(u32 btype)
{
	char cpu_arm1176sjz[] = "1176JZF-S";
	char db_men[] = "CX24XXX";
	char *cpu_s, *db_s;
	u32 chip_id = get_cpu_type();
   u32 chip_revid = get_cpu_rev();

	if( btype == BOARD_IRD )
	{
		cpu_s = cpu_arm1176sjz;
		db_s = db_men;
		printf("NXP Semiconductors. - %s SoC with ARM%s\n", db_s, cpu_s);
      printf("Chip:  %s %s\n", ((chip_id == NEVIS)?"NEVIS":"PECOS"), map_revid_string(chip_revid)); 
	}
	else
		printf( "unrecognized board [type=%u,cpu=%u]: no board info\n", btype, chip_id );
}

/*************************************************************************
 * get_board_rev() - setup to pass kernel board revision information
 *************************************************************************/
u32 get_board_rev(void)
{
	u32 rev = 0;
	u32 btype = get_board_type();

	if (btype == BOARD_IRD)
	{
		rev = 1;
	}
	return(rev);
}
