
#include "stbcfg.h"
#include <common.h>
#include <exports.h> /* malloc and free prototypes */

#include "basetype.h"
#include "tlv_image.h"
#include "zlib.h"

#define CNXT_STATUS 	int
#define CNXT_STATUS_BAD_PARAMETER	1
#define CNXT_STATUS_OK 			0

/* uncomment out ENABLE_TIME_MEASURE to print out the measurement data in console */
/*#define ENABLE_TIME_MEASURE*/

/******************/
/* EXTERN         */
/******************/
extern void FCopy(void *dest, const void *src, unsigned int count);
extern unsigned long get_timer_masked(void);
extern bool GetTopRam(u_int32 *puTopRamAddr); /* defined in board/nxp/common/ramc.c */

//#if defined(CONFIG_CMD_IMLS)
extern flash_info_t flash_info[]; /* info for FLASH chips */
//#endif

/* defined in /board/nxp/common/decarm.c */
extern struct tag_decarm decarm_tag; /* will be populate in load_tlv_image() */


/******************/
/* STATICs        */
/******************/
static bool load_tlv_image( 
               u_int32 *puTLVAddr, 
               u_int32 uImageType, 
               u_int32 **ppuImageEntry ); 
static CNXT_STATUS tlv_image_process_measure( 
               u_int32                     uImageType, 
               u_int32                     uTimeBegin,
               u_int32                     uTimeEnd,
               u_int32                     uImageLength,
               TLV_IMAGE_PROCESS_PHASE     Phase );

/**********************************************************************
 *
 * 
 *
 **********************************************************************/
void Int2Hex(int x, char *b) 
{
    int i;
    int j;
    
    *(b++) = '0';
    *(b++) = 'x';
    
    for (i = 28; i >= 0; i -= 4) {
         j = (x >> i) & 0xf;
         if (j < 10) {
             *(b++) = j + '0';
         } else {
             *(b++) = j + 'a' - 10;
         }
    }
    *b = 0;
}

/* convert an integer to string */
void Int2String( u_int32 uValue, char *pszString )
{
   u_int8  uLoop; 
   u_int8  uNumOfDigit;
   u_int32 uDigit;
   char szTemp[11];  /* the max integer of u_int32 has 10 digits total */
   
   uLoop       = 1; 
   uNumOfDigit = 0;
   do
   {
      uDigit = uValue % 10;
      szTemp[uLoop++] = '0' + uDigit;
      uValue = uValue /10; /* go to the next LSD digit */
      uNumOfDigit++;
   }
   while( uValue != 0 );
    
   /* return the string */
   for ( uLoop = uNumOfDigit; uLoop > 0; uLoop-- )
   {
      *(pszString++) = szTemp[uLoop];
   }
   *pszString = 0;
   
} /* Int2String */  
       
/**********************************************************************
 *
 * 
 *
 **********************************************************************/
u_int32 get_tlv(u_int32 *mn, int TLV_TYPE, int *len, void **p) 
{
   /* the magic number should be word aligned because its supposd
    * to be on a 32/64K boundary */
   if (*mn != TLV_MAGIC_NUMBER_INT) 
   {
      return 1;
   }

   mn++;
   while (1) 
   {
     if (*mn == TLV_TYPE) 
     {
        mn++;
        *len = *mn;
        mn++;
        *p = mn;
        return 1;
     } 
     else if (*mn != TLV_TYPE_IMAGE_DATA) 
     {
        mn++;
        mn = (u_int32*)((int)mn + *mn + 4);
     } 
     else 
     {
        return 0;
     }
   }
}


/**********************************************************************
 *
 * 
 *
 **********************************************************************/

void display_tlv_header(u_int32 *mn) 
{
    int len;
    void *p;

    if ( get_tlv(mn, TLV_TYPE_LENGTH, &len, &p) == 1 ) {
                printf( "     [TLV LENGTH] 0x%08x\n", *(int*)p);
    }
                
    if ( get_tlv(mn, TLV_TYPE_CRC, &len, &p) == 1 ) {
                printf( "     [TLV    CRC] 0x%08x\n", *(int*)p);
    }
    
    if ( get_tlv(mn, TLV_TYPE_DEST_ADDR, &len, &p) == 1 ) {
                printf( "     [TLV EntryP] 0x%08x\n", *(int*)p);
    }
    
    if ( get_tlv(mn, TLV_TYPE_END_ADDR, &len, &p) == 1 ) {
                printf( "     [TLV  EADDR] 0x%08x\n", *(int*)p);
    }
    
    if ( get_tlv(mn, TLV_TYPE_FLASH_ADDR, &len, &p) == 1 ) {
                printf( "     [TLV  FADDR] 0x%08x\n", *(int*)p);
    }
    
    if ( get_tlv(mn, TLV_TYPE_COPYRIGHT, &len, &p) == 1 ) {
                printf( "     [TLV   COPY] %s\n", (char*)p);
    
    }
                
    if ( get_tlv(mn, TLV_TYPE_IMAGE_TYPE, &len, &p) == 1 ) {
                printf( "     [TLV  ITYPE] 0x%08x\n", *(int*)p);
    }
    
    if ( get_tlv(mn, TLV_TYPE_DESCRIP, &len, &p) == 1 ) {
                printf( "     [TLV DESCRI] %s\n", (char*)p);
    }
    
    if ( get_tlv(mn, TLV_TYPE_BOOT_STRING, &len, &p) == 1 ) {
                printf( "     [TLV  BOOTS] %s\n", (char*)p);
    }
    
    if ( get_tlv(mn, TLV_TYPE_IMAGE_DATA, &len, &p) == 1 ) {
                printf( "     [TLV  IDATA] Length = 0x%08x\n", len);
    }
    
}

//#if defined(CONFIG_CMD_IMLS)
static u_int32 flash_base = CFG_FLASH_BASE;

/**********************************************************************
 *
 * 
 *
 **********************************************************************/

void list_tlv_images() {

    u_int32 *mn;
    u_int32 i;
    u_int32 flash_size = 0;  

    for (i = 0; i < CFG_MAX_FLASH_BANKS; ++i) 
    {
       flash_size += flash_info[i].size;
    }

    for (mn = (u_int32*)flash_base;
         mn < (u_int32*)(flash_base + flash_size); 
         mn += (TLV_BLOCK_MIN_SIZE)) {
        
        if (*mn == TLV_MAGIC_NUMBER_INT) {
            printf("TLV image in ROM at 0x%08X\n", (uint)mn);
            display_tlv_header(mn);
            /* The next iteration of the loop will increment the address
             * to the next block */
            mn = (u_int32*)((int)mn & ~0xFFFF);
            printf("\n");
        }
    }

    printf("End of image list\n");
}
//#endif

/* find the image type in a given TLV block */
bool get_image_type( u_int32 *puTLVAddr, u_int32 *puImageType ) 
{
   u_int32 uRetCode;
   int     len;
   void    *pData;
   
   /* check for NULL pointers */
   if ( (puTLVAddr == NULL) || (puImageType == NULL) )
   {
      return FALSE;
   }
   
   if (*puTLVAddr != TLV_MAGIC_NUMBER_INT) 
   {
      /* not TLV header */
      return FALSE;
   }
   
   uRetCode = get_tlv( puTLVAddr, TLV_TYPE_IMAGE_TYPE, &len, &pData);
   if ( uRetCode != 1 )
   {
      return FALSE;
   }
   
   *puImageType = *(u_int32*)pData; 
   
   return TRUE;
} /* find_image_type */

/* uImageType can be either compressed type or uncompressed type, this function will find either, and set the flag pbCompressed */
bool get_tlv_image_info( u_int32 *puTLVAddr, TLV_IMAGE_INFO *pImageInfo ) 
{
   int              rc;
   u_int32          *mn;
   void             *it;
   void             *daddr = 0;
   void             *crc;
   void             *prd_data = 0;
   void             *pRL = 0;
   void             *pSL = 0;
   int              len;
   u_int32          uImageTypeFound;
   u_int32          uImageLength;
   u_int32          image_checksum;
   u_int32          calculated_crc;
   u_int32          uImageType;
   u_int32          uTimeCRCBegin = 0;
   u_int32          uTimeCRCEnd   = 0;

   /* check for NULL pointers */
   if ( (puTLVAddr == NULL) || (pImageInfo == NULL) )
   {
      return FALSE;
   }

   uImageType = pImageInfo->uImageType;
   mn = puTLVAddr;

   if (*mn != TLV_MAGIC_NUMBER_INT) 
   {
      return FALSE;
   }

   rc = get_tlv(mn, TLV_TYPE_IMAGE_TYPE, &len, &it);
   if (rc != 1) 
   {
       return FALSE;
   }

   uImageTypeFound = *(u_int32*)it;
   if ( (uImageTypeFound & 0x0F) != (uImageType & 0x0F) ) /* look for either compressed or uncompressed */ 
   {
       /* not the type of image we're interested in... */
       return FALSE;
   }

   /* Get the destination address of the image */
   rc = get_tlv(mn, TLV_TYPE_DEST_ADDR, &len, &daddr);
   if (rc != 1) 
   {
       return FALSE;
   }
   daddr = (void*)(*(int*)daddr);
   pImageInfo->pDstAddr = daddr;

   /* Get the address where the compressed data starts */
   rc = get_tlv(mn, TLV_TYPE_IMAGE_DATA, &len, &prd_data);
   if (rc != 1) 
   {
       return FALSE;
   }
   uImageLength = (u_int32)len;
   pImageInfo->uImageLen = (u_int32)len;
   pImageInfo->pSrcAddr  = prd_data;


   /* Get the shared ram length and runtime ram length if available, only available for decarm image */
   pImageInfo->uSharedRAMSize  = 0; 
   pImageInfo->uRunTimeRAMSize = 0; 
   pImageInfo->uSharedRAMStart = 0;
   rc = get_tlv(mn, TLV_TYPE_SHARED_RAM_LENGTH, &len, &pSL);
   if( rc == 1 )
   {
      pImageInfo->uSharedRAMSize  = *(u_int32*)pSL;
   }
   rc = get_tlv(mn, TLV_TYPE_RUNTIME_RAM_LENGTH, &len, &pRL);
   if( rc == 1 )
   {
      pImageInfo->uRunTimeRAMSize = *(u_int32*)pRL;
   }

   /* for DECARM image, if the dst address is 0 or 0xFFFFFFFF, then boot loader will decide to load it at top of RAM */
   if ( uImageType == IMAGE_TYPE_COM_SLAVE || uImageType == IMAGE_TYPE_UNC_SLAVE )
   {
      /* round up the runtime size and shared ram size to be multiple of 1M */
      pImageInfo->uRunTimeRAMSize = (pImageInfo->uRunTimeRAMSize + DECARM_SHARED_RAM_ADDR_CONSTRAINS - 1) & ~(DECARM_SHARED_RAM_ADDR_CONSTRAINS -1);
      pImageInfo->uSharedRAMSize   = (pImageInfo->uSharedRAMSize + DECARM_SHARED_RAM_ADDR_CONSTRAINS - 1) & ~(DECARM_SHARED_RAM_ADDR_CONSTRAINS-1);

      if( (((u_int32)pImageInfo->pDstAddr) == (u_int32)NULL) || (((u_int32)pImageInfo->pDstAddr) == 0xFFFFFFFF) )
      {
         /* Decarm Memory configuration requirement                                    */
         /*   TOP RAM                                                                  */
         /*   |        | -> DestAddress + RunTime lenghth                              */
         /*   |        |                        }                                      */
         /*   .        .                         runTime RAM Length                    */
         /*   |        |                        }                                      */
         /*   |        | -> DestAddress = entry point                                  */
         /*   |        |                        }                                      */
         /*   .        .                        shared RAM Length                      */
         /*   |        |                        }                                      */
         /*   |        | ->DestAddress - Shared Ram Length = shard RAM Start address   */
         /*   |        |                                                               */
         /*   |        |                                                               */
         /*   |        |                                                               */
         /*   RAM 0x0                                                                  */
         u_int32 uTopRam = 0;

         GetTopRam( &uTopRam );
         pImageInfo->pDstAddr = (void*)(uTopRam - pImageInfo->uRunTimeRAMSize);
         /* Decarm Shared RAM starting address is at DstAddress - SharedRam Length */
         pImageInfo->uSharedRAMStart = (u_int32)pImageInfo->pDstAddr - pImageInfo->uSharedRAMSize;
      }
   }

   /* check CRC for uncopressed image */
   if (uImageTypeFound & 0x80)  
   {
       /* image is compressed */
       pImageInfo->bCompressed = TRUE;
   }
   else
   {
       /* image is not compressed */
       pImageInfo->bCompressed = FALSE;

       /* get the system time */
       uTimeCRCBegin = get_timer_masked();

       /* check CRC */
       rc = get_tlv(mn, TLV_TYPE_CRC, &len, &crc);
       image_checksum = *((unsigned int*)crc);
       if (rc != 1) 
       {
           printf("Error Image Data\n");
           return FALSE;
       }
       calculated_crc = crc32(0, (unsigned char *)prd_data, uImageLength);

       /* get the system time */
       uTimeCRCEnd = get_timer_masked( );

       /* get the delta and print out the measurement result */
       tlv_image_process_measure( 
          uImageType, uTimeCRCBegin, uTimeCRCEnd, uImageLength, CRC_CHECKING );      

       if (calculated_crc == image_checksum)
       {
          return TRUE;
       }
       printf("INVALID CHKSUM, Calculated CRC 0x%08X != Image CRC 0x%08X\n", 
              (unsigned int)calculated_crc, (unsigned int)image_checksum);
       return FALSE;
   }

   return TRUE;
}

#define ZALLOC_ALIGNMENT	16

static void *zalloc (void *x, unsigned items, unsigned size)
{
	void *p;

	size *= items;
	size = (size + ZALLOC_ALIGNMENT - 1) & ~(ZALLOC_ALIGNMENT - 1);

	p = malloc (size);

	return (p);
}

static void zfree (void *x, void *addr, unsigned nb)
{
	free (addr);
}


/* ===========================================================================
     Decompresses the source buffer into the destination buffer.  sourceLen is
   the byte length of the source buffer. Upon entry, destLen is the total
   size of the destination buffer, which must be large enough to hold the
   entire uncompressed data. (The size of the uncompressed data must have
   been saved previously by the compressor and transmitted to the decompressor
   by some mechanism outside the scope of this compression library.)
   Upon exit, destLen is the actual size of the compressed buffer.
     This function can be used to decompress a whole file at once if the
   input file is mmap'ed.

     uncompress returns Z_OK if success, Z_MEM_ERROR if there was not
   enough memory, Z_BUF_ERROR if there was not enough room in the output
   buffer, or Z_DATA_ERROR if the input data was corrupted.
*/

int uncompress (Bytef *dest, uLongf *destLen, const Bytef *source, uLong sourceLen)
{
    z_stream stream;
    int err;

    stream.next_in = (Bytef*)source;
    stream.avail_in = (uInt)sourceLen;
    /* Check for source > 64K on 16-bit machine: */
    if ((uLong)stream.avail_in != sourceLen) return Z_BUF_ERROR;

    stream.next_out = dest;
    stream.avail_out = (uInt)*destLen;
    if ((uLong)stream.avail_out != *destLen) return Z_BUF_ERROR;

    stream.zalloc = (alloc_func)zalloc;
    stream.zfree = (free_func)zfree;
    stream.outcb = NULL;

    err = inflateInit(&stream);
    if (err != Z_OK) return err;

    err = inflate(&stream, Z_FINISH);
    if (err != Z_STREAM_END) {
        inflateEnd(&stream);
        return err == Z_OK ? Z_BUF_ERROR : err;
    }
    *destLen = stream.total_out;

    err = inflateEnd(&stream);
    return err;
}



/**********************************************************************
 *  Find image attributes and load the image to RAM, return Image Entry
 *  point.
 *
 **********************************************************************/
static bool load_tlv_image( u_int32 *puTLVAddr, u_int32 uImageType, u_int32 **ppuImageEntry ) 
{
   int             dec_len; 
   TLV_IMAGE_INFO  ImageInfo;
   bool            bRetCode;
   u_int32         uTimeBegin = 0;
   u_int32         uTimeEnd   = 0;
   
   /* found the image type we are looking for */
   ImageInfo.uImageType = uImageType;
   ImageInfo.bCompressed = FALSE;
   ImageInfo.pDstAddr    = NULL;
   ImageInfo.pSrcAddr    = NULL;
   ImageInfo.uImageLen   = 0;
   
   /* get image attributes */
   bRetCode = get_tlv_image_info( puTLVAddr, &ImageInfo ); 
   if ( bRetCode != TRUE )
   {
      return bRetCode;
   }

   if( uImageType == IMAGE_TYPE_UNC_SLAVE || uImageType == IMAGE_TYPE_COM_SLAVE )
   {
      /* populate the global decarm_tag tag info */
      decarm_tag.code_size = (u32)ImageInfo.uRunTimeRAMSize;
      decarm_tag.code_start = (u32)(int*)ImageInfo.pDstAddr;
      decarm_tag.sharedram_size = (u32)ImageInfo.uSharedRAMSize;
      decarm_tag.sharedram_start = (u32)ImageInfo.uSharedRAMStart;
   }
   /* return the image entry point to branch to. */
   if (ppuImageEntry != NULL) 
   {
      *ppuImageEntry = (u_int32 *)ImageInfo.pDstAddr;
   }             
   
   /* get the start time before decompressing/copying */
   uTimeBegin = get_timer_masked();
   
   if ( ImageInfo.bCompressed == TRUE )   {
      /* Do the decompression */
      if (uncompress((Bytef*)ImageInfo.pDstAddr, (uLongf*)&dec_len, (Bytef*)ImageInfo.pSrcAddr, ImageInfo.uImageLen) != Z_OK ) 
      {
         printf("Error uncompressing image\n");
         return FALSE;
      } 
      else 
      {
         /* get the end time after decompressing */
         uTimeEnd = get_timer_masked();
         
         printf("Decompressed length = 0x%08X\n", dec_len);
         
         /* get the delta and print out the measurement result */
         tlv_image_process_measure( 
            uImageType, uTimeBegin, uTimeEnd, dec_len, DECOMPRESSING );      
      }
   } 
   else 
   {
      /* Stright copy... */
      FCopy((void*)ImageInfo.pDstAddr, (void*)ImageInfo.pSrcAddr, ImageInfo.uImageLen);
      
      /* get the end time after copying */
      uTimeEnd = get_timer_masked();
      /* get the delta and print out the measurement result */
      tlv_image_process_measure( 
         uImageType, uTimeBegin, uTimeEnd, ImageInfo.uImageLen, IMAGE_LOADING );      
   }
   
   return TRUE;
}

/*#if defined(CONFIG_CMD_IMLS)*/
/* look for the uImageType and load it in RAM */
bool find_tlv_image(u_int32 uImageType, u_int32 **ppTLVEntry) 
{
   bool    bRetCode;
   u_int32 uNumOfBlocks;
   u_int32 uLoop;
   u_int32 uImageTypeFound;
   u_int32 *mn;
   u_int32 flash_size = 0;  

   for (uLoop = 0; uLoop < CFG_MAX_FLASH_BANKS; ++uLoop) 
   {
      flash_size += flash_info[uLoop].size;
   }
   
   /* get the number of TLV blocks in the ROM */   
   uNumOfBlocks  = flash_size/TLV_BLOCK_MIN_SIZE;
    
   for ( uLoop = 0; uLoop < uNumOfBlocks; uLoop++ )
   {  
      /* get the start address of TLV block */
      mn = (u_int32*)(flash_base + (uLoop * TLV_BLOCK_MIN_SIZE));
      if (*mn == TLV_MAGIC_NUMBER_INT) 
      {
         bRetCode = get_image_type( mn, &uImageTypeFound );
         if ( bRetCode == FALSE )
         {
             /* should not get here*/
             return FALSE;
         }
         if ( (uImageTypeFound & 0x0F) == (uImageType & 0x0F) ) /* look for either compressed or uncompressed */ 
         {
             break;
         }
         /* not the type of image we're interested in, go to the next TLV block */
      }
   }
   
   if ( uLoop >= uNumOfBlocks )
   {
      /* didn't find it */
      return FALSE;
   }
   
   *ppTLVEntry = mn;        

   return TRUE;
}
/*#endif*/

/* look for the uImageType in ROM and load it in RAM */
bool find_and_load_tlv_image(u_int32 uImageType, u_int32 **ppuImageEntry) 
{
   u_int32 *pTLVEntry;
   bool    bRetCode;
   
   bRetCode = find_tlv_image( uImageType, &pTLVEntry );
   if ( bRetCode == FALSE )
   {
      return bRetCode;
   }
              
   /* Load the image to RAM and return the image entry point to branch to. */
   bRetCode = load_tlv_image( pTLVEntry, uImageType, ppuImageEntry ); 
   if ( bRetCode != TRUE )
   {
      return bRetCode;
   }
   
   return TRUE;
}

static CNXT_STATUS tlv_image_process_measure( 
               u_int32                     uImageType, 
               u_int32                     uTimeBegin,
               u_int32                     uTimeEnd,
               u_int32                     uImageLength,
               TLV_IMAGE_PROCESS_PHASE    Phase )
{
   u_int32 uTimeDelta = 0;
   u_int32 uRate      = 0; /* in megabytes per second */
   #ifdef ENABLE_TIME_MEASURE
   char *pszImageProcessPhase[] =
   {
      /*   0  0x00  */   "CRC CHECKING",
      /*   1  0x01  */   "DECOMPRESSING",
      /*   2  0x02  */   "IMAGE LOADING"
   }; 
   #endif  
   
   if ( Phase > TLV_IMAGE_PROCESS_PHASE_LAST )
   {
      /* wrong phase passed */
      return CNXT_STATUS_BAD_PARAMETER;
   }

   uTimeDelta = uTimeEnd - uTimeBegin;
   if ( uTimeDelta )
   {    
      uRate = uImageLength/uTimeDelta;
   }

   #ifdef ENABLE_TIME_MEASURE
   /* print out the info */
   printf("Time measurement for image type %d, during phase %s\n", (int)uImageType, (char *)pszImageProcessPhase[Phase]);
   printf("     Time used = %dus, Data rate = %dMB/s\n", (int)uTimeDelta, (int)uRate);
   #endif

   return CNXT_STATUS_OK;
} /* tlv_image_process_measure */








