
#include "stbcfg.h"

#include <common.h>
#include <asm/io.h>
#include <config.h>
#include <asm-arm/arch-arm1176/sys_proto.h>
#include <i2c.h>
#include <asm-arm/mach-types.h>

#include "tlv_image.h"
#include "status_codes.h"

#if (ISPECOS)
#define MACH_TYPE               0x483
#else
#define MACH_TYPE               0x4C8
#endif

/* default atag list address */
u32 ATAG_ADDRESS = 0x00030000;

/* defined in /board/nxp/common/decarm.c */
extern struct tag_decarm decarm_tag; /* will be populate once boot_decarm is called */
extern void boot_decarm (void);

extern CNXT_STATUS cnxt_gpio_set_output_level(u_int32 uGPIO, bool bOn);

extern bool get_meminfo( ulong *pMemInfo, ulong *puNumOfEntries );

#ifdef BOARD_LATE_INIT
/* setup_xxx_tag() are defined in lib_arm/bootm.c */
extern void setup_start_tag (bd_t *bd);

# ifdef CONFIG_SETUP_MEMORY_TAGS
extern void setup_memory_tags (bd_t *bd);
# endif

#ifdef CONFIG_CMDLINE_TAG
extern void setup_commandline_tag (bd_t *bd, char *commandline);
#endif

#ifdef CONFIG_ETHERNET_TAG
extern void setup_ethernet_tags(bd_t *bd);
#endif 

#ifdef CONFIG_DECARM_TAG
extern void setup_decarm_tag (bd_t *bd, struct tag_decarm *decarm );
#endif 

#ifdef CONFIG_CONIN_TAG
extern void setup_conin_tag (bd_t *bd);
#endif 

extern void setup_end_tag (bd_t *bd);
#endif /* #ifdef BOARD_LATE_INIT */

/*******************************************************
 * Routine: delay
 * Description: spinning delay to use before udelay works
 ******************************************************/
void raise(void)
{
}

void __aeabi_unwind_cpp_pr0 (void)
{
}

static inline void delay (unsigned long loops)
{
	__asm__ volatile ("1:\n" "subs %0, %1, #1\n"
		"bne 1b":"=r" (loops):"0" (loops));
}

#if defined(CONFIG_SHOW_BOOT_PROGRESS)
void show_boot_progress(int progress)
{
	printf("Boot reached stage %d\n", progress);
}
#endif


/*****************************************
 * Routine: board_init
 * Description: Early hardware init.
 *****************************************/
int board_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;
	gd->bd->bi_arch_number = MACH_TYPE;
	gd->bd->bi_boot_params = ATAG_ADDRESS; 
	return 0;
}

/*******************************************************
 * Routine: misc_init_r
 * Description: Init ethernet (done here so udelay works)
 ********************************************************/
int misc_init_r (void)
{
   #if ( PIO_MII0_RESET != GPIO_INVALID )
   if( cnxt_gpio_set_output_level( PIO_MII0_RESET, TRUE ) != CNXT_STATUS_OK )
   {
      printf( "EMAC0 reset failed!!!, 0x%x\n", PIO_MII0_RESET ); 
   }
   #endif

   #if ( PIO_MII1_RESET != GPIO_INVALID )
   if( cnxt_gpio_set_output_level( PIO_MII1_RESET, TRUE ) != CNXT_STATUS_OK )
   {
      printf( "EMAC0 reset failed!!!, 0x%x\n", PIO_MII1_RESET ); 
   }
   #endif

	return 0;
}

/**********************************************
 * Routine: dram_init
 * Description: sets uboots idea of sdram size
 **********************************************/
int dram_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;
	u32 mtype, btype;
   u32 meminfo[4]; /* 1st is size, 2nd is start address, 3rd is size of second entry, etc */
   ulong count = 2; /* number of entries, each entry has 2 words.*/

	btype = get_board_type();
	mtype = get_mem_type();

	display_board_info(btype);

   get_meminfo( (u_int32*)meminfo, &count );

	gd->bd->bi_dram[0].size = meminfo[0];
	gd->bd->bi_dram[0].start = meminfo[1];
   #if ( CONFIG_NR_DRAM_BANKS > 1 )
   {
      if( count > 1 )
      {
         gd->bd->bi_dram[1].size = meminfo[2];
         gd->bd->bi_dram[1].start = meminfo[3];
      }
      else
      {
         gd->bd->bi_dram[1].size = 0;
         gd->bd->bi_dram[1].start = gd->bd->bi_dram[0].start + gd->bd->bi_dram[0].size;
      }
   }
   #endif

	return 0;
}

/**********************************************
 * Routine: board_late_init
 * Description: Init Rom descriptors and  bring 
 * DECARM out of reset, this
 * function must be called after ram is initialized.
 **********************************************/
#ifdef BOARD_LATE_INIT

// AIX_3RDPARTY_CODE_CHANGED: 
#ifdef AIX_METZ_BOOT_PROTOCOL
extern int metz_startup( void );
#endif

int board_late_init (void)
{
   DECLARE_GLOBAL_DATA_PTR;
#ifdef CONFIG_CMDLINE_TAG
	char *commandline = getenv ("bootargs");
#endif
	bd_t	*bd = gd->bd;

   /* boot decarm and populate the decarm image info if neccessary */
   boot_decarm();

   /* setup ATAG list so that image loaded via debugger can run */
   setup_start_tag(bd);

#ifdef CONFIG_SETUP_MEMORY_TAGS
   setup_memory_tags (bd);
#endif

#ifdef CONFIG_ETHERNET_TAG
	setup_ethernet_tags (bd);
#endif

#ifdef CONFIG_DECARM_TAG
   if( decarm_tag.code_size && decarm_tag.code_start )
   {
      setup_decarm_tag (bd, &decarm_tag );
   }
#endif

// AIX_3RDPARTY_CODE_CHANGED: disable console input capability
#ifdef CONFIG_CONIN_TAG
   setup_conin_tag (bd);
#endif

#ifdef CONFIG_CMDLINE_TAG
	setup_commandline_tag (bd, commandline);
#endif

	setup_end_tag (bd);

   #if (I2C_CONFIG_EEPROM_ADDR != NOT_PRESENT)
   {
      extern u_int32 ReadCdgEepromC(u_int32 uSize, u_int32 uOffset);
      u32 temp;
      u32 ramspeed_eeprom;
      u32 ramspeed_actual;
   
      /* check MC speed against EEPROM MC CONFIG, and make sure the MC speed is expected */
      temp = ReadCfgEepromC(4, 0);
      /* the fist byte is version */
      if( (temp&0xFF) >=3 )
      {
         /* MC speed is at offset 0x21 */
         temp = ReadCfgEepromC(4, 0x21);
         ramspeed_eeprom = 310;
         if( (temp&0xFF) >= 4 ) /* 800 parts start at index 4 */
         {
            ramspeed_eeprom = 333;
         }
         /* MC speed is not linear, so have to use hardcoded value to determine the speed */
         switch ( (CNXT_GET( MC_PLL_MEM_CTRL2_REG, 0xFFFFFFFF)) & 0xFF )
         {
            case 0x43:           ramspeed_actual = 333;     break;
            case 0x3E:           ramspeed_actual = 310;     break;
            case 0x42:
            default:             ramspeed_actual = 330;     break;
         }
         if( ramspeed_actual != ramspeed_eeprom )
         {
            printf("McSpeed actual %dMhz != EEPROM CONFIG %dMhz\n", ramspeed_actual, ramspeed_eeprom );
         }
         else
         {
            printf("McSpeed:%d Mhz\n", ramspeed_actual );
         }
      }
   }
   #endif /* I2C_CONFIG_EEPROM_ADDR != NOT_PRESENT */

// AIX_3RDPARTY_CODE_CHANGED: 
#ifdef AIX_METZ_BOOT_PROTOCOL
  if( metz_startup() > 0 )
  {
    setenv( "bootcmd", NULL );
  }
#endif
	return 0;
}
#endif /* #ifdef BOARD_LATE_INIT */


