/**
*******************************************************************************
**
** \file      cmd_metz.c
**
** \version   01.00.00
**
** \date      20.03.2009
**
** \author    Juergen Sachs
**
** \brief     U-Boot command for Metz
**
** \attention THIS CODE IS PROVIDED AS IS. AIXSOLVE ACCEPTS NO RESPONSIBILITY
**            OR LIABILITY FOR ANY ERRORS OR OMMISIONS
**
** Copyright (c) 2009 AixSolve GmbH (Germany)
**
*******************************************************************************
*/

//#include "stbcfg.h"

#include <common.h>
#include <command.h>

#include <status_codes.h>

#define TEST_PROTOCOL 0
#define DISABLE_INPUT_WITH_CREDENTIALS 1
#define BOOTSTRAP_BACKDOOR 0

/*---------------------------------------------------------------------------*/

#include <ns16550.h>
#include <serial.h>

static void init_port( NS16550_t port, int baudrate )
{
  int divisor = (CFG_NS16550_CLK + 8 * baudrate)
              / (16 * baudrate);
  NS16550_reinit( port, divisor );
}

static int flush_port( int port )
{
  int i = 0;

  printf( "--- flushing port %d: ", port );
  while( _serial_tstc( port+1 ) && i++ < 50 )
  {
    printf( "%02x", _serial_getc( port+1 ) );
  }
  printf( "\n" );
  return 0;
}

static int get_port( int port, int timeout )
{
  do {
    if( _serial_tstc( port+1 ) )
      return _serial_getc( port+1 );
    udelay(1000);
  } while( --timeout > 0 );
  return -1;
}

static void put_port( int port, int c )
{
  _serial_putc_raw( c, port+1 );
}

extern CNXT_STATUS cnxt_gpio_get_level( int uGPIO, int * bOn );

#if 0 // brauchen wir das? Wozu?
static void send_metz_id( void )
{
  static const int           port = 0;
  static const unsigned char id[] = { // !!! keep this ID constant
    0x06, 0, 15,                      // neither the version nor other chars 
    0xB8, 0x00, 0x01,                 // is allowed to change
    'M','2','H',' ','V',' ','0','.','0','0',' ', 0x00
  };

  const unsigned char * p;
  const unsigned char * p_end;
  unsigned char crc = 0;

  for( p = id, p_end = id + sizeof( id )
     ; p < p_end
     ; p++
     )
  {
    put_port( port, *p );
    crc += *p;
  }
  put_port( port, crc );
}
#endif

/*---------------------------------------------------------------------------*/

static int wait_metz_protocoll( void )
{ 
  static const unsigned char request[] = 
     { 0xAA,0x06,0x00,0x08,0x00,0x00,0x0B,0x2F,0xFF,0xFF,0xFF,0x0E };
  static const unsigned char response[] = 
     { 0xAA,0x12,0x00,0x08,0x00,0x00,0x24,0x50,0xFF,0xFF,0xFE,0xC8 };
  
  static const char * port_name[3] = { "eserial0", "eserial1", "eserial2" };
  int           c, rc, port;
  const unsigned char *  p;
  const unsigned char *  p_end;

  //init_port( (NS16550_t)CFG_NS16550_COM1, 115200 );
  //init_port( (NS16550_t)CFG_NS16550_COM2, 115200 );
  // COM3 already initialized for shell

  serial_reinit_all();
  flush_port( 0 );
  flush_port( 1 );
  flush_port( 2 );

  printf( "[mld] waiting for 0xAA...\n" );

  do {
    for( port = 0; port < 3; port++ )
    {
      c = get_port( port, 100 ); 
      if( -1 != c )
      {
#if TEST_PROTOCOL
        if( c == 'a' ) // TODO: debug, force protocol on UART, remove later
        {
          c = request[0];
          goto Response;
        }
#else
        if( c != request[0] )
        {
          printf( "[mld] invalid request start at port %d. (0x%02X)\n", port+1, c );
          rc = 0; goto Exit;
        }
#endif
        break;
      }
    }
  } while( c != request[0] );

  for( p = request + 1, p_end = request + sizeof( request )
     ; p < p_end
     ; p++
     )
  {
    c = get_port( port, 100 );
    if( c != *p )
    {
      printf( "[mld] unknown request byte: %02x, expected[%d]: %02x\n"
            , c, p - request, *p
            );
      rc = 0; goto Exit;
    }
  }
  
//Response:
  printf( "[mld] request at UART %d OK - sending response\n", port+1 );
  serial_assign( port_name[port] );
  for( p = response, p_end = response + sizeof( response )
     ; p < p_end
     ; p++
     )
  {
    put_port( port, *p );
  }

  rc = port+1;

Exit:
  return rc;  
}

static int check_input_disabled( void )
{
  unsigned long * sfbase = (unsigned long *)0xF0080000;
  if( *sfbase != 0xFFFFFFFF )
  {
    printf( "***I: secure flash occupied (%08lx)\n", *sfbase );
#if DISABLE_INPUT_WITH_CREDENTIALS
    return 1;
#endif
  }
  return 0;
}

extern int NS16550_input_disabled; // ns16550.c

int metz_startup( void )
{
  int tmp;

//  printf( "### random number: %08x\n", *(volatile unsigned long*)0xE0500398 );
  if( check_input_disabled() )
  {
#if BOOTSTRAP_BACKDOOR
    cnxt_gpio_get_level( 27, &tmp );
    if( !tmp )
    {
      printf( "***W: bootstrap GPIO low: input not disabled\n" );
    }
    else
#endif
    {
      printf( "***I: input disabled\n" );
      NS16550_input_disabled = 1;
    }
    return 0;
  }

#if TEST_PROTOCOL
  tmp = 0;
#else
  cnxt_gpio_get_level( 27, &tmp );
#endif
  if( 0 == tmp )
  {
    return wait_metz_protocoll();
  }

  return 0;
}

/**
*******************************************************************************
**
** \brief 
**
** description
**
** \param 
** \param 
** \return 
**
*******************************************************************************
**/
int do_metz(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
  switch( argc )
  {
  case 0: case 1:
    printf( "Usage:\n%s\n", cmdtp->help );
    return 1;

  case 2:
    if( strncmp( argv[1], "info", 4 ) == 0 )
    {
      printf( "%s\n", "---- info ----\n" );
      return 0;
    }

    printf( "Usage:\n%s\n", cmdtp->help );
    return 1;

  default:
    return 1;
  }

  return 0;
}

U_BOOT_CMD(
  metz, 4, 1, do_metz
  , "metz      - Metz command set\n"
  , "metz info - show some info\n"
);
