/*
 *  U-Boot command for OneNAND support
 *
 *  Copyright (C) 2005-2007 Samsung Electronics
 *  Kyungmin Park <kyungmin.park@samsung.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <common.h>
#include <command.h>

#include <linux/mtd/compat.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/nfc.h>

#include <asm/io.h>

extern struct mtd_info nfc_mtd;
extern struct nfc_dev nfc_dev; 

int do_nfc(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
	int ret = 0;

	switch (argc) {
	case 0:
	case 1:
		printf("Usage:\n%s\n", cmdtp->help);
		return 1;

	case 2:
		if (strncmp(argv[1], "init", 4) == 0) {
			nfc_init();
			return 0;
		}
		if (strncmp(argv[1], "info", 4) == 0) {
         printf("%s\n", nfc_mtd.name);
		   return 0;
	   }
	   printf("Usage:\n%s\n", cmdtp->help);
		return 1;

	default:
		/* At least 4 args */
		if (strncmp(argv[1], "erase", 5) == 0) {
         struct erase_info instr = {
            .callback	= NULL,
			};
			ulong start, end;
			ulong block;
			char *endtail;

			if (strncmp(argv[2], "block", 5) == 0) {
				start = simple_strtoul(argv[3], NULL, 10);
				endtail = strchr(argv[3], '-');
				end = simple_strtoul(endtail + 1, NULL, 10);
			} else {
				start = simple_strtoul(argv[2], NULL, 10);
				end = simple_strtoul(argv[3], NULL, 10);

				start >>= nfc_dev.erase_shift;
				end >>= nfc_dev.erase_shift;
				/* Don't include the end block */
				end--;
			}

			if (!end || end < 0)
				end = start;

			printf("Erase block from 0x%lx to 0x%lx\n", start, end);

			for (block = start; block <= end; block++) {
				instr.addr = block << nfc_dev.erase_shift;
				instr.len = 1 << nfc_dev.erase_shift;
				ret = nfc_erase(&nfc_mtd, &instr);
				if (ret) {
					printf("erase failed at block 0x%lx, ret=%d\n", block, ret);
					break;
				}
			}

			printf("Done.[ret=%d]\n", ret);
			return 0;
		}

		if (strncmp(argv[1], "read", 4) == 0) {
			ulong addr = simple_strtoul(argv[2], NULL, 16);
			ulong ofs = simple_strtoul(argv[3], NULL, 16);
			size_t len = simple_strtoul(argv[4], NULL, 16);
			size_t retlen = 0;
			int oob = strncmp(argv[1], "read.oob", 8) ? 0 : 1;

			if (oob)
				ret = nfc_read_oob(&nfc_mtd, ofs, len, &retlen, (u_char *) addr);
			else
				ret = nfc_read(&nfc_mtd, ofs, len, &retlen, (u_char *) addr);

			printf("Done.[ret=%d]\n", ret);
			return 0;
		}

		if (strncmp(argv[1], "write", 5) == 0) {
			ulong addr = simple_strtoul(argv[2], NULL, 16);
			ulong ofs = simple_strtoul(argv[3], NULL, 16);
			size_t len = simple_strtoul(argv[4], NULL, 16);
			size_t retlen = 0;
         printf("Writing %d \n",len); 
			ret = nfc_write(&nfc_mtd, ofs, len, &retlen, (u_char *) addr);

			printf("Done.[ret=%d]\n", ret);
			return 0;
		}

		if (strncmp(argv[1], "block", 5) == 0) {
			ulong addr = simple_strtoul(argv[2], NULL, 16);
			ulong block = simple_strtoul(argv[3], NULL, 10);
			ulong page = simple_strtoul(argv[4], NULL, 10);
			size_t len = simple_strtol(argv[5], NULL, 10);
			size_t retlen = 0;
			ulong ofs;
			int oob = strncmp(argv[1], "block.oob", 9) ? 0 : 1;

			ofs = block << nfc_dev.erase_shift;
			if (page)
				ofs += page << nfc_dev.page_shift;

			if (!len) {
				if (oob)
					len = 64;
				else
					len = 512;
			}

			if (oob)
				ret = nfc_read_oob(&nfc_mtd, ofs, len, &retlen, (u_char *) addr);
			else
				ret = nfc_read(&nfc_mtd, ofs, len, &retlen, (u_char *) addr);

			printf("Done.[ret=%d]\n", ret);
			return 0;
		}

		break;
	}

	return 0;
}

U_BOOT_CMD(
	nfc,	7,	1,	do_nfc,
	"nfc     - NXP NAND flash sub-system\n",
   "nfc init - [re]initialize the NAND flash controller\n"
	"nfc info - show available NAND flash devices\n"
	"nfc read[.oob] <addr> <ofs> <len> - read data at ofs with len to addr\n"
	"nfc write <addr> <ofs> <len> - write data at ofs with len from addr\n"
	"nfc erase [block] <saddr>[-]<eaddr> - erase from start addr to end addr\n"
	"nfc block[.oob] <addr> <block> [page] [len] - read data from (block [, page]) to addr\n"
);
