/*
 * CPU specific code
 */

#include "stbcfg.h"
#include "basetype.h"
#include "../../board/nxp/common/miic.h"
#include "status_codes.h"
#include "mgpio.h"

#include <config.h>
#include <common.h>
#include <command.h>

/*
 * External Function Prototypes 
 */
extern u_int32 GetChipID(void);
extern u_int32 GetChipRev(void);
extern MIIC_RETURNCODE MIIC_Initialize(void );

/*
 * Global Data 
 */
unsigned long  ChipID, ChipRevID;

/* read co-processor 15, register #1 (control register) */
static unsigned long read_p15_c1 (void)
{
	unsigned long value;

	__asm__ __volatile__(
		"mrc	p15, 0, %0, c1, c0, 0@ read control reg\n"
		: "=r" (value)
		:
		: "memory");
	return value;
}

/* write to co-processor 15, register #1 (control register) */
static void write_p15_c1 (unsigned long value)
{
	__asm__ __volatile__(
		"mcr	p15, 0, %0, c1, c0, 0@ write it back\n"
		:
		: "r" (value)
		: "memory");

	read_p15_c1 ();
}

static void cp_delay (void)
{
	volatile int i;


	for (i = 0; i < 100; i++);
}

/* See also ARM Ref. Man. */
#define C1_MMU		(1<<0)		/* mmu off/on */
#define C1_ALIGN	(1<<1)		/* alignment faults off/on */
#define C1_DC		(1<<2)		/* dcache off/on */
#define C1_WB		(1<<3)		/* merging write buffer on/off */
#define C1_BIG_ENDIAN	(1<<7)		/* big endian off/on */
#define C1_SYS_PROT	(1<<8)		/* system protection */
#define C1_ROM_PROT	(1<<9)		/* ROM protection */
#define C1_IC		(1<<12)		/* icache off/on */
#define C1_HIGH_VECTORS	(1<<13)	/* location of vectors: low/high addresses */
#define RESERVED_1	(0xf << 3)	/* must be 111b for R/W */


int cpu_init (void)
{
  int i;

   /*****************************/
   /* Get Chip and Chip Rev IDs */
   /*****************************/
   ChipID    = GetChipID();
   ChipRevID = GetChipRev();

	/*****************************/
	/* Clear out GPIO registers. */
	/*****************************/
  // AIX_3RDPARTY_CODE_CHANGED:js: alter Code hat nur Bank 0 gesetzt, 
  // DRIVE_REGs auf 0 setzten ist ziemlicher Quatsch!
  for( i = 0; i < NUM_GPI_BANKS; i++ )
  {
    *(LPREG)(GPI_POS_EDGE_REG +i*GPI_BANK_SIZE) = 0;
    *(LPREG)(GPI_NEG_EDGE_REG +i*GPI_BANK_SIZE) = 0;
    *(LPREG)(GPI_DRIVE_OFF_REG+i*GPI_BANK_SIZE) = 0xFFFFFFFF;
  }

#define SET_GPIO_DRIVE_LOW_REG(b,v) *(LPREG)(GPI_DRIVE_LOW_REG + ((b)*GPI_BANK_SIZE)) = v
#define SET_GPIO_DRIVE_HIGH_REG(b,v) *(LPREG)(GPI_DRIVE_HIGH_REG + ((b)*GPI_BANK_SIZE)) = v

#ifdef GPI_DRIVE_LOW_REG_0_DEFAULT
  SET_GPIO_DRIVE_LOW_REG( 0, GPI_DRIVE_LOW_REG_0_DEFAULT );
#endif
#ifdef GPI_DRIVE_LOW_REG_1_DEFAULT
  SET_GPIO_DRIVE_LOW_REG( 1, GPI_DRIVE_LOW_REG_1_DEFAULT );
#endif
#ifdef GPI_DRIVE_LOW_REG_2_DEFAULT
  SET_GPIO_DRIVE_LOW_REG( 2, GPI_DRIVE_LOW_REG_2_DEFAULT );
#endif
#ifdef GPI_DRIVE_LOW_REG_3_DEFAULT
  SET_GPIO_DRIVE_LOW_REG( 3, GPI_DRIVE_LOW_REG_3_DEFAULT );
#endif
#ifdef GPI_DRIVE_LOW_REG_4_DEFAULT
  SET_GPIO_DRIVE_LOW_REG( 4, GPI_DRIVE_LOW_REG_4_DEFAULT );
#endif
#ifdef GPI_DRIVE_LOW_REG_5_DEFAULT
  SET_GPIO_DRIVE_LOW_REG( 5, GPI_DRIVE_LOW_REG_5_DEFAULT );
#endif

#ifdef GPI_DRIVE_HIGH_REG_0_DEFAULT
  SET_GPIO_DRIVE_HIGH_REG( 0, GPI_DRIVE_HIGH_REG_0_DEFAULT );
#endif
#ifdef GPI_DRIVE_HIGH_REG_1_DEFAULT
  SET_GPIO_DRIVE_HIGH_REG( 1, GPI_DRIVE_HIGH_REG_1_DEFAULT );
#endif
#ifdef GPI_DRIVE_HIGH_REG_2_DEFAULT
  SET_GPIO_DRIVE_HIGH_REG( 2, GPI_DRIVE_HIGH_REG_2_DEFAULT );
#endif
#ifdef GPI_DRIVE_HIGH_REG_3_DEFAULT
  SET_GPIO_DRIVE_HIGH_REG( 3, GPI_DRIVE_HIGH_REG_3_DEFAULT );
#endif
#ifdef GPI_DRIVE_HIGH_REG_4_DEFAULT
  SET_GPIO_DRIVE_HIGH_REG( 4, GPI_DRIVE_HIGH_REG_4_DEFAULT );
#endif
#ifdef GPI_DRIVE_HIGH_REG_5_DEFAULT
  SET_GPIO_DRIVE_HIGH_REG( 5, GPI_DRIVE_HIGH_REG_5_DEFAULT );
#endif

	init_gpio_extenders();

   #if PIO_CORE_VDD_LEVEL != GPIO_INVALID
   /* bit 22 in register 0xe044014c determines whether to drive PIO_CORE_VDD_LEVEL high or low.
      bit 22 = 0: High Core voltage(1.15v Nominal).  drive pio high 
      bit 22 = 1: Low Core voltage(1.10v Nomimal), drive pio low */
   if( (CNXT_GET( SREG_FUSE_CODE_REG, SREG_FUSE_FUNC_CORE_VDD_MASK )) == SREG_FUSE_FUNC_CORE_VDD_HIGH )
   {
      /* Core VDD high */
      cnxt_gpio_set_output_level( PIO_CORE_VDD_LEVEL, TRUE ); 
   }
   else
   {
      /* Core VDD low */
      cnxt_gpio_set_output_level( PIO_CORE_VDD_LEVEL, FALSE ); 
   }
   #endif /* PIO_CORE_VDD_LEVEL != GPIO_INVALID */

	/*
	* Initialize the IIC driver
	*/
	MIIC_Initialize();


 #if SERIAL1 == INTERNAL_UART3
	/*
	* Use UART 3
	*/
  #if PLL_PIN_GPIO_MUX0_REG_DEFAULT == NOT_DETERMINED
	*((LPREG)PLL_PIN_GPIO_MUX0_REG) |= (PLL_PIN_GPIO_MUX0_PIO_14 | PLL_PIN_GPIO_MUX0_PIO_15);
  #endif
 #else
	/*
	* Use UART 1
	*/
  #if PLL_PIN_GPIO_MUX0_REG_DEFAULT == NOT_DETERMINED
	*((LPREG)PLL_PIN_GPIO_MUX0_REG) |= (PLL_PIN_GPIO_MUX0_PIO_1 | PLL_PIN_GPIO_MUX0_PIO_2);
  #endif
#endif

	return 0;
}

int cleanup_before_linux (void)
{
	/*
	 * this function is called just before we call linux
	 * it prepares the processor for linux
	 *
	 * we turn off caches etc ...
	 */

	unsigned long i;

   /* turn off the eth PHY  */
   #ifdef CONFIG_RTL8201
	{
		extern void cnxt_terminate (void);
		cnxt_terminate ();
	}
   #endif

	disable_interrupts ();

	/* turn off I/D-cache */
	asm ("mrc p15, 0, %0, c1, c0, 0":"=r" (i));
	i &= ~(C1_DC | C1_IC);
	asm ("mcr p15, 0, %0, c1, c0, 0": :"r" (i));

	/* flush I/D-cache */
	i = 0;
	asm ("mcr p15, 0, %0, c7, c7, 0": :"r" (i));  /* invalidate both caches and flush btb */
	asm ("mcr p15, 0, %0, c7, c10, 4": :"r" (i)); /* mem barrier to sync things */
	return(0);
}


void reset_cpu (ulong addr)
{
   /* softreset ARM */
	*(LPREG)RST_SOFTRESET_REG = RST_SOFTRESET;
   return;

}

int do_reset (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	disable_interrupts ();
	reset_cpu (0);
	/*NOTREACHED*/
	return(0);
}

void icache_enable (void)
{
	ulong reg;

	reg = read_p15_c1 ();	/* get control reg. */
	cp_delay ();
	write_p15_c1 (reg | C1_IC);
}

void icache_disable (void)
{
	ulong reg;

	reg = read_p15_c1 ();
	cp_delay ();
	write_p15_c1 (reg & ~C1_IC);
}

int icache_status (void)
{
	return(read_p15_c1 () & C1_IC) != 0;
}

void dcache_enable(void)
{
	ulong reg;

	reg = read_p15_c1 ();	/* get control reg. */
	cp_delay ();
	write_p15_c1 (reg | C1_DC);
}

void dcache_disable(void)
{
	ulong reg;

	reg = read_p15_c1 ();
	cp_delay ();
	write_p15_c1 (reg & ~C1_DC);
}

int dcache_status(void)
{
	return(read_p15_c1 () & C1_DC) != 0;
}
