/****************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                  */
/*                       SOFTWARE FILE/MODULE HEADER                        */
/*                      Conexant Systems Inc. (c) 2004                      */
/*                         All Rights Reserved                              */
/****************************************************************************/
/*
 * Filename:        gpioext.c
 *
 *
 * Description:     Low-level boot loader GPIO extender routines
 *
 *
 * Author:          Tim White
 *
 ****************************************************************************/
/* $Id: gpioext.c 55536 2008-09-12 17:08:25Z swartzwg $
 ****************************************************************************/

/******************/
/* Include Files  */
/******************/
#include "stbcfg.h"
#include "cnxt_types.h"

/***************/
/* Global Data */
/***************/
HW_DWORD CNXT_PIO_EXP_REG_SHADOW;

#if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO)
#define CNXT_PIO_EXP_REG PCI_ISA_DEVICE_BASE(CNXT_PIO_EXP_DESCRIPTOR)
#endif

/********************************************************************/
/*  init_gpio_extenders                                             */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Initialize the GPIO extenders.                              */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      None.                                                       */
/*                                                                  */
/*  NOTES:                                                          */
/*     This is a very nasty function that hardcodes the chip select */
/*     used and the descriptor set up yet allows the user to        */
/*     specify the address of the device (which is fixed by the     */
/*     choice of descriptor!). ToDo: Fix up the hardware config     */
/*     file to allow the chip select to be specified and NOT the    */
/*     device address if a GPIO extender is in use.                 */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Nothing.                                                    */
/********************************************************************/
void init_gpio_extenders(void)
{
   #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO)
   LPREG pPrimaryDesc;
   LPREG pSecondaryDesc;
  /*
   * Setup the desired descriptor to access the GPIO extender using 
   * values specified in the hardware configuration file.
   */
   pPrimaryDesc   = (LPREG)PCI_ISA_PRIMARY_DESC_ADDR(CNXT_PIO_EXP_DESCRIPTOR);
   pSecondaryDesc = (LPREG)PCI_ISA_SECONDARY_DESC_ADDR(CNXT_PIO_EXP_DESCRIPTOR);
   
  *pPrimaryDesc   = (0xc74a0000 | 
                         (CNXT_PIO_EXP_CHIP_SELECT<<PCI_ISA_DESC_CS_SHIFT));
  *pSecondaryDesc = 0x9209;

  /*
   * Set the default GPIO extender value (NOTE: This is write-only)
   */
  CNXT_PIO_EXP_REG_SHADOW  = DEFAULT_PIO_EXP_REG_VALUE;

  /*
   * Now write the shadow to the write-only register
   */
  *(LPREG)CNXT_PIO_EXP_REG = CNXT_PIO_EXP_REG_SHADOW;
  #endif  
}

/********************************************************************/
/* IMPORTANT NOTE:                                                  */
/*                                                                  */
/* We must use shadow copies of these registers rather than reading */
/* the I2C extender part. Reading the part switches all pins to     */
/* input mode automatically and, as a result, does not guarantee to */
/* return the state that was last written to the part. As a specific*/
/* example, pin 0 on extender 1 is wired to the Modem OffHook LED   */
/* on Klondike boards and reading it always returns 0 regardless of */
/* the state of the LED due to the external wiring.                 */
/********************************************************************/

/********************************************************************/
/*  FUNCTION:    read_gpio_extender                                 */
/*                                                                  */
/*  PARAMETERS:  None                                               */
/*                                                                  */
/*  DESCRIPTION: Return the current state of the I2C GPIO extender  */
/*                                                                  */
/*  RETURNS:     Internal variable shadowing state of part.         */
/*                                                                  */
/********************************************************************/
u_int8 read_gpio_extender(void)
{
    return (CNXT_PIO_EXP_REG_SHADOW);
}

/********************************************************************/
/*  FUNCTION:    iicWriteGpioExtender                               */
/*                                                                  */
/*  PARAMETERS:  bit_mask - indicating which bits to set/clear      */
/*               value    - indicating the value of those bits      */
/*                                                                  */
/*  DESCRIPTION: Set the state of the I2C GPIO extender             */
/*                                                                  */
/*  RETURNS:     Previous state of the device.                      */
/*                                                                  */
/********************************************************************/
u_int8 write_gpio_extender(u_int8 bit_mask, u_int8 value)
{
    u_int8 bState;

    /*
     * Get the existing state
     */
    bState = CNXT_PIO_EXP_REG_SHADOW;

    /*
     * Set states of appropriate GPIO lines
     */
    CNXT_PIO_EXP_REG_SHADOW = (CNXT_PIO_EXP_REG_SHADOW & ~bit_mask) |
                                                 (value & bit_mask);

    /*
     * Write the new state to the device
     */
#if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO)
   *(LPREG)CNXT_PIO_EXP_REG = CNXT_PIO_EXP_REG_SHADOW;
#else    
#if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IIC)
    MIIC_Write(I2C_ADDR_NIM_EXT, (u_int8*)&CNXT_PIO_EXP_REG_SHADOW, 1, I2C_BUS_NIM_EXT);
#endif
#endif

    return(bState);
}

