/****************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                  */
/*                       SOFTWARE FILE/MODULE HEADER                        */
/*                   Conexant Systems Inc. (c) 1998-2004                    */
/*                            All Rights Reserved                           */
/****************************************************************************/
/*
 * Filename:       MGPIO.H
 *
 *
 * Description:    Low Level GPIO access function prototypes
 *
 *
 * Author:         Dave Wilson
 *
 ****************************************************************************/
/* $Id: mgpio.h 55536 2008-09-12 17:08:25Z swartzwg $   
 ****************************************************************************/

#ifndef _MGPIO_H_
#define _MGPIO_H_

/* Determine the address of the ISA GPIO expander if present */
#if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO)
#define CNXT_PIO_EXP_REG PCI_ISA_DEVICE_BASE(CNXT_PIO_EXP_DESCRIPTOR)
#endif

/* Convert a virtual GPIO number to a KAL interrupt ID. Note that this macro
   only applies when the GPIO is an internal pin */
#define INT_ID_FROM_GPIO(gpio) (CALC_INT_ID(PIC_GPIO, (gpio)&GPIO_MASK_PIN_NUMBER))

/*******************************/
/* Exported function prototype */
/*******************************/

/********************************************************************/
/*  FUNCTION:    cnxt_gpio_set_output_level                         */
/*                                                                  */
/*  PARAMETERS:  uGPIO    - General GPIO identifier for the pin     */
/*                          whose logic level is to be set.         */
/*               bOn      - TRUE to set logic level 1, FALSE for 0  */
/*                                                                  */
/*  DESCRIPTION: This function sets the state of an individual GPIO */
/*               output to logical 1 or 0. The output level is      */
/*               determined based upon whether the GPIO is marked   */
/*               GPIO_NEGATIVE_POLARITY or GPIO_POSITIVE_POLARITY.  */
/*                                                                  */
/*  RETURN VALUES:                                                  */
/*               CNXT_STATUS_OK,                                    */
/*               CNXT_STATUS_INVALID_DEVICE,                        */
/*               CNXT_STATUS_INVALID_NUMBER,                        */
/*               CNXT_STATUS_INVALID_TYPE                           */
/*                                                                  */
/*  CONTEXT:     Interrupt safe if the GPIO passed is NOT connected */
/*               to an I2C GPIO extender on the target board.       */
/*                                                                  */
/********************************************************************/
CNXT_STATUS cnxt_gpio_set_output_level(u_int32 uGPIO, bool bOn);

/********************************************************************/
/*  FUNCTION:    cnxt_gpio_get_level                                */
/*                                                                  */
/*  PARAMETERS:  uGPIO    - General GPIO identifier for the pin     */
/*                          whose logic level is to be set.         */
/*               pbOn     - Pointer to storage for returned logic   */
/*                          level.                                  */
/*                                                                  */
/*  DESCRIPTION: This function returns the current value of a GPIO  */
/*               pin. If it is an output, the last value written    */
/*               will be returned, else the value will indicate the */
/*               level on an input pin. The logic level is written  */
/*               to the address pointed to by pbOn.                 */
/*                                                                  */
/*               Note that the returned logic level takes into      */
/*               account the state of the GPIO polarity bit in the  */
/*               generic GPIO descriptor passed to the function.    */
/*                                                                  */
/*  RETURN VALUES:                                                  */
/*               CNXT_STATUS_OK,                                    */
/*               CNXT_STATUS_INVALID_DEVICE,                        */
/*               CNXT_STATUS_INVALID_NUMBER,                        */
/*               CNXT_STATUS_INVALID_TYPE,                          */
/*               CNXT_STATUS_BAD_PTR                                */
/*                                                                  */
/*  CONTEXT:     Interrupt safe if the GPIO passed is NOT connected */
/*               to an I2C GPIO extender on the target board.       */
/*                                                                  */
/********************************************************************/
CNXT_STATUS cnxt_gpio_get_level(u_int32 uGPIO, bool *pbOn);

/********************************************************************/
/*  FUNCTION:    cnxt_gpio_set_input                                */
/*                                                                  */
/*  PARAMETERS:  uGPIO    - General GPIO identifier for the pin     */
/*                          which is to be set as an input.         */
/*                                                                  */
/*  DESCRIPTION: This function sets the supplied GPIO pin as an     */
/*               input assuming this is possible.                   */
/*                                                                  */
/*  RETURN VALUES:                                                  */
/*               CNXT_STATUS_OK,                                    */
/*               CNXT_STATUS_INVALID_DEVICE,                        */
/*               CNXT_STATUS_INVALID_NUMBER,                        */
/*               CNXT_STATUS_INVALID_TYPE,                          */
/*               CNXT_STATUS_INVALID_STATE                          */
/*                                                                  */
/*  CONTEXT:     Interrupt safe if the GPIO passed is NOT connected */
/*               to an I2C GPIO extender on the target board.       */
/*                                                                  */
/********************************************************************/
CNXT_STATUS cnxt_gpio_set_input(u_int32 uGPIO);


#endif /* _MGPIO_H_ */

/****************************************************************************
 * Modifications:
 * $Log$
 *
 ****************************************************************************/ 

