// ***************************************************************
// *       GNU style assembly code generated from mmu.s          *
// ***************************************************************
// * This file is automatically generated by the build process   *
// * from the ARM style assembly file. Editing it is, therefore, *
// * completely futile!  DO NOT EDIT THIS FILE!!!                *
// ***************************************************************
//***************************************************************************
//                   CONEXANT PROPRIETARY AND CONFIDENTIAL                  
//                       SOFTWARE FILE/MODULE HEADER                       
//                      Conexant Systems Inc. (c) 2004
//                               Austin, TX                              
//                          All Rights Reserved       
//***************************************************************************
//*                                                                          
//* Filename:       mmu.s                                           
//*                                                                          
//* Description:    ARM V4 MMU Support                                      
//*                                                                          
//* Author:         Dave Moore                                               
//*                                                                          
//***************************************************************************
//* $Id$
//****************************************************************************

//******************/
//* Include Files  */
//******************/
#include "stbcfg.asmh"

#include "cpu.asmh"
#include "board.asmh"

#include "mmu.asmh"


//-------------------------------------------------
//                Functions
//-------------------------------------------------


#if 0
.text 1
.align 4
#else 
.text 1
.align 4
#endif 


//
//   FUNCTION: mmuSetPageTabBase
// 
//   PARAMETERS: r0 - 16K aligned page table beginning address
// 
//   DESCRIPTION: set the page table base register via CP15 register 2 
// 
//   RETURNS: r0 : 1 - OK
//                 0 - FAILED (bad input address alignment)
//
//   REGS USED: r0, r1, r2
// 

    .global mmuSetPageTabBase
mmuSetPageTabBase: 

#if MMU_CACHE_DISABLE == 1

    mov r0, #0x1 // fake an OK exit

#else //  MMU_CACHE_DISABLE = 0

    // Check for proper alignment of address
    // Comment for PECOS
    // This mask depends on the TLB Control Register (c2, c0, 2) for ARM1176 / V6 architecture.
    // The TLB CR is zero by reset for full backward compatibiity reasons (v5 or ARM920). 
    // And the mask can be changed by altering the TLB CR.
    ldr r2, =0x3fff
    ands r1, r0, r2

    // Load page table address and set success return
    mcreq p15, 0, r0, c2, c0, 0
    moveq r0, #0x1 // OK exit
    movne r0, #0x0 // FAILED exit

#endif // IF MMU_CACHE_DISABLE = 0
 
    bx lr


//
//   FUNCTION: mmuGetPageTabBase
// 
//   PARAMETERS: None
// 
//   DESCRIPTION: get the page table base register via CP15 register 2 
// 
//   RETURNS: Contents of CP15 Reg 2 in r0
//
//   REGS USED: r0
//

    .global mmuGetPageTabBase
mmuGetPageTabBase: 

#if MMU_CACHE_DISABLE == 1

    mov r0, #0x0 // bogus return

#else // MMU_CACHE_DISABLE = 0

    mrc p15, 0, r0, c2, c0, 0

#endif // MMU_CACHE_DISABLE 

    bx lr


// 
//  FUNCTION: mmuSetDomainAccessControl
// 
//  PURPOSE: Sets the domain access-control register via CP15 register 3 
// 
//  PARAMETERS: r0 - value to be placed in CP15 reg 3 (see description below) 
//
//  DESCRIPTION:
//
//   MMU accesses are primarily controller thru the use of domains. There are
//   16 domains and each has a 2 bit field to access it. Two types of users are
//   supported, client and manager. The domains are defined in the domain access
//   control register.
//
//   INTERPRETING DOMAIN ACCESS CONTROL REGISTER
//
//   31 29 27 25 23 21 19 17 15 13 11 09 07 05 02 01      Bit#
//   +-----------------------------------------------+
//   |15|14|13|12|11|10|09|08|07|06|05|04|03|02|01|00|    Domain#
//   +-----------------------------------------------+
//
//    Value   Meaning     Description
//    -----   -------     -----------
//     00     No Access   Any access generates Domin fault
//     01     Client      Accesses are checked against Access Permission (AP)
//                        bits in the section or page descriptor
//     10     Reserved    Reserved. Currently behaves like the No Access mode.
//     11     Manager     Access are not checked against the access permission bits
//                        so a permission fault cannot be generated.
//
//  RETURNS: nothing
// 
//  REGS USED: r0
//
      .global mmuSetDomainAccessControl
mmuSetDomainAccessControl: 

    mcr p15, 0, r0, c3, c0, 0

    bx lr




// 
//  FUNCTION: mmuGetDomainAccessControl()
// 
//  PURPOSE: Gets the domain access-control register contents via CP15 register 3 
// 
//  PARAMETERS: None 
// 
//   MMU accesses are primarily controller thru the use of domains. There are
//   16 domains and each has a 2 bit field to access it. Two types of users are
//   supported, client and manager. The domains are defined in the domain access
//   control register.
//
//   INTERPRETING DOMAIN ACCESS CONTROL REGISTER
//
//   31 29 27 25 23 21 19 17 15 13 11 09 07 05 02 01      Bit#
//   +-----------------------------------------------+
//   |15|14|13|12|11|10|09|08|07|06|05|04|03|02|01|00|    Domain#
//   +-----------------------------------------------+
//
//    Value   Meaning     Description
//    -----   -------     -----------
//     00     No Access   Any access generates Domin fault
//     01     Client      Accesses are checked against Access Permission (AP)
//                        bits in the section or page descriptor
//     10     Reserved    Reserved. Currently behaves like the No Access mode.
//     11     Manager     Access are not checked against the access permission bits
//                        so a permission fault cannot be generated.
//
//  RETURNS: r0 = contents of CP15 register 3
// 
//  REGS USED: r0
//
      .global mmuGetDomainAccessControl
mmuGetDomainAccessControl: 

      mrc p15, 0, r0, c3, c0, 0 

      bx lr


// 
//   FUNCTION: mmuDrainWriteBuffer
// 
//   PURPOSE: drains write buffer (execution stops until complete)
//
//   PARAMETERS: None
//
//   RETURNS: Nothing
//
//   REGS USED: r0
//
    .global mmuDrainWriteBuffer
mmuDrainWriteBuffer: 

#if MMU_CACHE_DISABLE == 0

    mov r0, #0
    mcr p15, 0, r0, c7, c10, 4

#endif // IF MMU_CACHE_DISABLE = 0

    bx lr


//
//   FUNCTION: mmuInvalidateTLB
//
//   PURPOSE: Invalidate Data & Instruction TLB's
//
//   PARAMETERS: None.
//
//   RETURNS: Nothing
//
//   REGS USED: r0
//

     .global mmuInvalidateTLB
mmuInvalidateTLB: 

#if MMU_CACHE_DISABLE == 0

     mov r0, #0
     mcr p15, 0, r0, c8, c7, 0

#endif // IF MMU_CACHE_DISABLE = 0

     bx lr


// 
//   FUNCTION: mmuInvalidateITLB
//
//   PURPOSE: Invalidate Instruction TLB
//
//   PARAMETERS: None. 
//
//   RETURNS: Nothing
//
//   REGS USED: r0
//

     .global mmuInvalidateITLB
mmuInvalidateITLB: 

#if MMU_CACHE_DISABLE == 0

     mov r0, #0
     mcr p15, 0, r0, c8, c5, 0

#endif // IF MMU_CACHE_DISABLE = 0

     bx lr


// 
//   FUNCTION: mmuInvalidateDTLB
//
//   PURPOSE: Invalidate Data TLB
//
//   PARAMETERS: None
//
//   RETURNS: Nothing
//
//   REGS USED: r0
//

     .global mmuInvalidateDTLB
mmuInvalidateDTLB: 

#if MMU_CACHE_DISABLE == 0

     mov r0, #0
     mcr p15, 0, r0, c8, c6, 0 

#endif // IF MMU_CACHE_DISABLE = 0

     bx lr


// 
//   FUNCTION: mmuInvalidateITLBEntry
//
//   PURPOSE: Invalidate single Instruction TLB entry
//
//   PARAMETERS:  r0 = MVA to be invalidated
//
//   RETURNS: Nothing
//
//   REGS USED: r0
//

    .global mmuInvalidateITLBEntry
mmuInvalidateITLBEntry: 

#if MMU_CACHE_DISABLE == 0

    mcr p15, 0, r0, c8, c5, 1 

#endif // IF MMU_CACHE_DISABLE = 0

    bx lr


// 
//   FUNCTION: mmuInvalidateDTLBEntry
//
//   PURPOSE: Invalidate single Data TLB entry
//
//   PARAMETERS:  r0 = MVA to be invalidated
//
//   RETURNS: Nothing
//
//   REGS USED: r0
//

    .global mmuInvalidateDTLBEntry
mmuInvalidateDTLBEntry: 

#if MMU_CACHE_DISABLE == 0

    mcreq p15, 0, r0, c8, c6, 1 

#endif // IF MMU_CACHE_DISABLE = 0

    bx lr

//     /*  Use of the below two APIs is deprecated. Would continue to exist 
//         for Trinity based solutions. All future chips should use 
//         Context ID register instead.*/

#if MMU_TYPE == MMU_TYPE_920T
// 
//   FUNCTION: mmuGetFCSE
// 
//   PURPOSE: return the FastContextSwitchExtension (register 13)
//
//   PARAMETERS: None
//
//   RETURNS: r0 = contents of CP15 Reg 13
//
//   REGS USED: r0
//

    .global mmuGetFCSE
mmuGetFCSE: 

    mrc p15, 0, r0, c13, c0, 0 

    bx lr


//
//   FUNCTION: mmuSetFCSE
// 
//   PURPOSE: Set the FastContextSwitchExtension (register 13)
//
//   PARAMETERS: r0 = FCSE PID in bits 31:25 (bits 24:0 SBZ)
//
//   RETURNS: r0 = 0 - SUCCESS
//                 1 - FAIL (bad PID) 
//
//   REGS USED: r0, r1, r2
//

    .global mmuSetFCSE
mmuSetFCSE: 

#if MMU_CACHE_DISABLE == 0

    // check for valid input
    ldr r2, =0x1ffffff
    ands r1, r0, r2

    mcreq p15, 0, r0, c13, c0, 0 
    moveq r0, #0x0 // load SUCCESS for return
    movne r0, #0x1 // load FAIL for return

#endif // MMU_CACHE_DISABLE

    bx lr

#else 

// 
//   FUNCTION: mmuGetContextID
// 
//   PURPOSE: return the Context ID Register (register c13, 1)
//
//   PARAMETERS: None
//
//   RETURNS: r0 = contents of CP15 Reg c13, 1
//
//   REGS USED: r0
//
 .global mmuGetContextID
mmuGetContextID: 
#if MMU_TYPE == MMU_TYPE_1176
      nop
#endif 
    mrc p15, 0, r0, c13, c0, 1 
    bx lr


//
//   FUNCTION: mmuSetContextID
// 
//   PURPOSE: Set the Context ID Register (register c13, 1)
//
//   PARAMETERS: r0 = {ProcID,ASID}  [31:8,7:0]
//
//   RETURNS: Nothing
//
//   REGS USED: r0
//
 .global mmuSetContextID
mmuSetContextID: 
#if MMU_CACHE_DISABLE == 0

    mcr p15, 0, r0, c13, c0, 1 

#endif // MMU_CACHE_DISABLE

    bx lr

#endif // IF MMU_TYPE = MMU_TYPE_920T
 
//
//   FUNCTION: mmuGetState
//
//   PARAMETERS: None
//
//   Returns: r0 = contents of CP15 Control Register (register 1)
//
//   31 30 29 28 27 26 25 24 23 22 21 20 19 18  17 16 15 14 13 12 11 10 9 8 7 6  4 3 2 1 0
//  +-------------------------------------------------------------------------------------+
//  | SBZ |FA|TR| SBZ |EE|VE|XP|U |FI| SBZ |IT|SBZ|DT|L4|RR| V| I| Z| F|R|S|B| SBO|W|C|A|M|
//  +-------------------------------------------------------------------------------------+
//
// 
//   REGS USED: r0
//
    .global mmuGetState
mmuGetState: 
  
    mrc p15, 0, r0, c1, c0, 0 

    bx lr


//
//   FUNCTION: mmuSetState
//
//   PARAMTERS: r0 - mask of the following:
//
//   31 30 29 28 27 26 25 24 23 22 21 20 19 18  17 16 15 14 13 12 11 10 9 8 7 6  4 3 2 1 0
//  +-------------------------------------------------------------------------------------+
//  | SBZ |FA|TR| SBZ |EE|VE|XP|U |FI| SBZ |IT|SBZ|DT|L4|RR| V| I| Z| F|R|S|B| SBO|W|C|A|M|
//  +-------------------------------------------------------------------------------------+
//
//   RETURNS: r0 = 0 - FAILED, 1 - SUCCESS
//
//

    .global mmuSetState
mmuSetState: 

#if MMU_CACHE_DISABLE == 1

       mov r0, #0x1 // bogus SUCCESS return

#else // MMU_CACHE_DISABLE = 0
#if (MMU_TYPE == MMU_TYPE_1176)
      ldr r2, =CP15_CNTL_REG_C0_AND_MASK
      and r1, r0, r2
      ldr r2, =CP15_CNTL_REG_C0_OR_MASK
      orr r0, r1, r2
      mcr p15, 0, r0, c1, c0, 0
      mov r0, #0x1

#else //(MMU_TYPE = MMU_TYPE_920T)
    //Load r2 with the bit pattern appropriate to the CPU type
       ldr r2, =0x3fff8c00 // isolate bits 29:15,11:10 (ARM920)

       // do some checks for valid input..

       and r1, r0, #0x78 // isolate bits 6:3
       cmp r1, #0x78 // they should all be set to 1
       andeqs r1, r0, r2 // they should all be zeroes
                                     // error exit, bits 29:15,11:10 not all 0s
       mcreq p15, 0, r0, c1, c0, 0 // OK, change CP15 Reg 1
       moveq r0, #0x1 // normal exit
       movne r0, #0x0 // error exit
#endif //(MMU_TYPE = MMU_TYPE_920T)
#endif // MMU_CACHE_DISABLE 

       bx lr

    .end 

//****************************************************************************
//* $Log$
//* 
//****************************************************************************



