#include <version.h>
#include <asm/sizes.h>
#include <config.h>


#include "cpu.asmh"
#include "stbcfg.asmh"

#define REMAP_MASK (0xF000FFFF)

#if( OVERRIDE_PHYS_RAM_BASE == YES )
#define UBOOT_STACK_RAM_TOP (0x000a0000+RAM_BASE_OVERRIDE)
#else
#define UBOOT_STACK_RAM_TOP 0x000a0000
#endif

.data 0
.align 4

/* _armboot_start should be UBOOT_STACK_RAM_TOP from a software config file. */
.globl _armboot_start
_armboot_start:
        .word 0


.text 0
.align 4

.globl _start
_start: 
	/* Interrupt vector table, located at 0xF0000000 */
	b	reset                                /* 0x000  Reset Vector     */   
	ldr	pc, _undefined_instruction           /* 0x004  Undef Vector     */
	ldr	pc, _software_interrupt              /* 0x008  SWI Vector       */
	ldr	pc, _prefetch_abort                  /* 0x00C  IAbort Vector    */
	ldr	pc, _data_abort                      /* 0x010  DAbort Vector    */
	ldr	pc, _not_used                        /* 0x014  Reserved Vector  */
	ldr	pc, _irq                             /* 0x018  IRQ Vector       */
	ldr	pc, _fiq                             /* 0x01C  FIQ Vector       */

	/* Flash Header */
	.word 0x00000000                             /* 0x020  Offset of Boot Signature                 */ 
	.word 0x00000000                             /* 0x024  Offset of Skip Region                    */ 
	.word 0x00000000                             /* 0x028  Skip Region Size (in 32-bit words)       */ 
	.word 0x00000000                             /* 0x02C  Reserved                                 */ 
	.word 0x00000000                             /* 0x030  Offset of RSA Public Key                 */ 
	.word 0x00000000                             /* 0x034  Offset of SSP SRAM Image                 */ 
	.word 0x00000000                             /* 0x038  Offset of Signed Padded Key 1            */ 
	.word 0x00000000                             /* 0x03C  Offset of Key 2 Secure Packet            */ 
	.word 0x00000000                             /* 0x040  SSP SRAM Image Size                      */ 
	.word 0x00000000                             /* 0x044  Offset of Pre-Validation Secure Packet   */ 
	.word 0x00000000                             /* 0x048  External Flash Image Size                */ 
	.word 0x00000000                             /* 0x04C  Offset of CM Ecrypted RSA Private Key    */ 
	.word 0x00000000                             /* 0x050  Default Content Protection Region Start  */ 
	.word 0x00000000                             /* 0x054  Default Content Protection Region End    */ 

_undefined_instruction: .word undefined_instruction                                                 
_software_interrupt:	.word software_interrupt                                                    
_prefetch_abort:	.word prefetch_abort
_data_abort:		.word data_abort
_not_used:		.word not_used
_irq:			.word irq
_fiq:			.word fiq
_pad:			.word 0x12345678

.global _end_vect
_end_vect:

	.balignl 16,0xdeadbeef

/* Byte addresses 0x80-0x83 in the 0th page of NAND flash hold parameters reserved 
 * for NAND ROM interface operation during NAND boot configuration. */
.global _nand_boot_params
_nand_boot_params:
   .word 0x00000000

.globl vector_start
vector_start:
	.word _start

.globl vector_end
vector_end:
	.word _end_vect

/*  
 * These are defined in the board-specific linker script. 
 */

.globl _relocate_dst_start
_relocate_dst_start:
	.word __relocate_dst_start

.globl _relocate_dst_end
_relocate_dst_end:
	.word __relocate_dst_end

.globl _bss_start
_bss_start:
	.word __bss_start

.globl _bss_end
_bss_end:
	.word __bss_end

#ifdef CONFIG_USE_IRQ
/* IRQ stack memory (calculated at run-time) */
.globl IRQ_STACK_START
IRQ_STACK_START:
	.word 0x0badc0de

/* IRQ stack memory (calculated at run-time) */
.globl FIQ_STACK_START
FIQ_STACK_START:
	.word 0x0badc0de
#endif

.align 4

/**************************************************************
 *                                                            
 * This is code actually starts                  
 *                                                            
 **************************************************************/
reset:

waithere:
	/* Use the following for debugging the u-boot out of initial reset */
//	b 	waithere

	b 	ddr_init		
after_ddr_init:
	
	bl 	InitializeSectons	/* copy rodata, data, bss section to ram + initialize stacks */


	/* _armboot_start is UBOOT_STACK_RAM_TOP defined in the beginning of this file */ 
	ldr     r1, =UBOOT_STACK_RAM_TOP
	ldr     r0, =_armboot_start
	str 	r1, [r0]

	bl 	InitializeStacks	/* setup stack */
	ldr 	pc, =pc_to_ram		/* remaps PC from 0xFXXXXXXX to Ram address */
pc_to_ram:

	b 	early_cnxt_init		/* Code taken from codeldr.S */
after_cnxt_init:
	b 	start_armboot		/* start C code */

/********************************************************************
 *
 *      This code has been taken as is from codeldr.S
 *      We have broken the routine into two parts, ddr_init and
 *	early_cnxt_init
 *
 ********************************************************************/
ddr_init:
	bl      ssp_workaround 
 
	// Ensure that wayward ASX accesses cannot hang the system completely.
	ldr 	r0, =HSX_ASX_BRIDGE_CTRL_REG
	ldr 	r4, [r0]
	bic 	r4, r4, #HSX_ASX_BRIDGE_CTRL_TIMEOUT_MASK
	orr 	r4, r4, #HSX_ASX_BRIDGE_CTRL_TIMEOUT
	str 	r4, [r0]

	// Remap: This remaps PC to 0xF0000000 address.
	// Pecos: In case of Pecos and later chips, the execution begins at
	//        address range 0xFFFFXXXX. The ASX bridge remaps the access from
	//        0xFFFFXXXX to 0xF000XXXX.
	// Note:  However if we dont do this, and pc might step above 64kb (which
	//        exactly is space b/w 0xFFFFXXXX & 0xF000XXXX.
	// Trinity: In case of Trinity based processor the execution begins at
	//        address 0x0 and memory interface remaps it to access from
	//        0x0XXXXXXX to 0xFXXXXXXX.
	//        We need to disable these remappings.

	mov     r0, pc
	ldr     r1, =REMAP_MASK
	and     r0, r0, r1
	add     r0, r0, #0x18
	mov     pc, r0
	nop
	nop
	nop


	// Initialize the GPIO pin mux and alt func registers for unique
	// IRD configuration.
#if (PLL_CONFIG0_REG_DEFAULT != NOT_DETERMINED)
	ldr 	r4, =PLL_CONFIG0_REG_DEFAULT
	ldr 	r0, =PLL_CONFIG0_REG
	str 	r4, [r0]
#endif 

	ldr 	r4, =PLL_PIN_ALT_FUNC_REG_DEFAULT
	ldr 	r0, =PLL_PIN_ALT_FUNC_REG
	str 	r4, [r0]
	
	ldr 	r4, =PLL_PIN_GPIO_MUX0_REG_DEFAULT
	ldr 	r0, =PLL_PIN_GPIO_MUX0_REG
	str 	r4, [r0]

	ldr 	r4, =PLL_PIN_GPIO_MUX1_REG_DEFAULT
	ldr 	r0, =PLL_PIN_GPIO_MUX1_REG
	str 	r4, [r0]
	
	ldr 	r4, =PLL_PIN_GPIO_MUX2_REG_DEFAULT
	ldr 	r0, =PLL_PIN_GPIO_MUX2_REG
	str 	r4, [r0]

	ldr 	r4, =PLL_PIN_GPIO_MUX3_REG_DEFAULT
	ldr 	r0, =PLL_PIN_GPIO_MUX3_REG
	str 	r4, [r0]
	
	ldr 	r4, =PLL_PIN_GPIO_MUX4_REG_DEFAULT
	ldr 	r0, =PLL_PIN_GPIO_MUX4_REG
	str 	r4, [r0]

	ldr 	r4, =PLL_PIN_GPIO_MUX5_REG_DEFAULT
	ldr 	r0, =PLL_PIN_GPIO_MUX5_REG
	str 	r4, [r0]
	
	// Initialize the secondary pin mux registers
	ldr 	r4, =SREG_2ND_PIN_MUX0_REG_DEFAULT
	ldr 	r0, =SREG_2ND_PIN_MUX0_REG
	str 	r4, [r0]

	ldr 	r4, =SREG_2ND_PIN_MUX1_REG_DEFAULT
	ldr 	r0, =SREG_2ND_PIN_MUX1_REG
	str 	r4, [r0]

	ldr 	r4, =SREG_2ND_PIN_MUX2_REG_DEFAULT
	ldr 	r0, =SREG_2ND_PIN_MUX2_REG
	str 	r4, [r0]

	ldr 	r4, =SREG_2ND_PIN_MUX3_REG_DEFAULT
	ldr 	r0, =SREG_2ND_PIN_MUX3_REG
	str 	r4, [r0]

	ldr 	r4, =SREG_2ND_PIN_MUX4_REG_DEFAULT
	ldr 	r0, =SREG_2ND_PIN_MUX4_REG
	str 	r4, [r0]

	ldr 	r4, =SREG_2ND_PIN_MUX5_REG_DEFAULT
	ldr 	r0, =SREG_2ND_PIN_MUX5_REG
	str 	r4, [r0]

/* Skip overriding settings of this register, 
 * if we are booting from NAND flash */
#if (CNXT_ROM_DESC_0_EARLY_OVERRIDE != 0)
   ldr   r0, =PCI_ROM_DESC0_REG
   ldr   r4, [r0]
   tst   r4, #0xC0 //PCI_ROM_DESC_NOR_NAND_MODE_MASK
   bne  booting_from_nand_flash
   ldr   r4, =CNXT_ROM_DESC_0_EARLY_OVERRIDE
   str 	r4, [r0]
booting_from_nand_flash:
#endif 
	
	// Initialize PLLs.
	bl 	SetPLLs


	//Initialize checkpoint output for all output devices.
	bl 	InitCheckpointOutput

	// Remove any remapping.
	bl 	RemoveFlashShadow

	// Clear the V bit.  This tells the ARM to look at 0x00000000
	// for the exception vector table.
	mrc 	p15, 0, r0, c1, c0
	and 	r0, r0, #0xffffdfff
	mcr 	p15, 0, r0, c1, c0

	// Initialize SDRAM controller.
	bl 	SetupDDRController

	// Set up ASB mode register
	bl 	SetUpASBModeReg

	// Enable the ARM11 Peripheral Port Remap register and set the non-shared device memory
	// This will make the L2 cache registers accessible
	ldr 	r0, =PERI_PORT_REMAP_REG_VALUE
	MCR 	p15, 0, r0, c15, c2, 4
	b	after_ddr_init

/********************************************************************/
early_cnxt_init:

    
	// Enable the I cache.
	bl 	mmuGetState
	bic	r0, r0, #CACHE_PROT_ENABLE
	bic 	r0, r0, #CACHE_DCACHE_ENABLE
	orr 	r0, r0, #CACHE_ICACHE_ENABLE
	orr 	r0, r0, #CACHE_FASTBUS | CACHE_ASYNC
	bl 	mmuSetState

	b	after_cnxt_init


/******************************************************************* 
 *  ssp_workaround                                                  *
 *                                                                  *
 *  PARAMETERS:                                                     *
 *      None.                                                       *
 *                                                                  *
 *  DESCRIPTION:                                                    *
 *      This code segment is a workaround for SSP PLL issue when    *
 *      SSP goes off the weed after reset if SSP is programmed      *
 *      to run 150Mhz speed in Pecos/Nevis revB .                   *
 *      The workaround of the issue is to softreset the chip, then  *
 *      SSP seems to function correctly.                            *
 *                                                                  *
 *  RETURNS:                                                        *
 *      Nothing.                                                    *
 *                                                                  *
 *  REGISTER USE:                                                   *
 *      r0, r1                                                      *
 ********************************************************************/

#define SSP_ENABLED_MASK    0x00000010
#define SSP_PLL_SPEED_MASK  0x18000000
#define SSP_PLL_SPEED_60MHZ 0x00000000
#define SSP_PLL_NOT_BYPASS_MASK 0x00001000

ssp_workaround:
	// check whether SSP is running or not, if not running, skip workaround.
	// the bit 4 in 0xe0441070 indicates running or not , 1 = running.
	ldr     r0, =0xe0441070
	ldr     r1, [r0]
	and     r0, r1, #SSP_ENABLED_MASK
	cmp     r0, #SSP_ENABLED_MASK
    	// SSP is not running, skip the workaround
	bne     SSPPLL_OK

	// check whether SSP PLL is running at 60 MHz or not, if 60 MHz, skip workaround.
	// the bits 28:27 == 0 in 0xe0441070 indicates running at 60 MHz.
	and     r0, r1, #SSP_PLL_SPEED_MASK
	cmp     r0, #SSP_PLL_SPEED_60MHZ
	// SSP is running at 60 MHz, skip the workaround
	beq     SSPPLL_OK

	// check the softreset bit in reset status register, if it is set, it means
	// softreset has been performed to get SSP alive, don't need to
	// pull SSP PLL not_bypass bit.
	ldr     r0, =RST_STATUS_REG
	ldr     r1, [r0]
	and     r1, r1, #RST_STATUS_SOFT_MASK
	cmp     r1, #RST_STATUS_SOFT_MASK
	beq     SSPPLL_OK
	
waitforSSPPLLSet:
	ldr      r0, =0xe0440070
	ldr      r1, [r0]
	and      r1, r1, #SSP_PLL_NOT_BYPASS_MASK
	cmp      r1, #SSP_PLL_NOT_BYPASS_MASK
	// loop if not_bypass bit is not set
	bne      waitforSSPPLLSet

	// not_bypass bit is set, do a chip softreset
	ldr      r0, =RST_SOFT_RESET_REG
	str      r1, [r0]

SSPPLL_OK:
	bx 	lr


/*******************************************************************
 *  SetUpASBModeReg                                                 
 *                                                                  
 *      Sets up the ASB mode register, needs to be executed early.  
 *                                                                  
 *  NOTE: Uses r0 and r1                                              
 *                                                                  
 *******************************************************************/
SetUpASBModeReg: 
	// In debug builds, always turn off PIT instruction prefetching.
	// Prefetch helps performance but causes problems when running
	// the ARM debuggers. Since they don't know how to flush the
	// buffer, you can get into states where the debugger tries to 
	// clear or step over a breakpoint but the instruction is actually
	// in the prefetch buffer rather than cache or SDRAM and so is not
	// affected. At best, this prevents you from stepping past a 
	// breakpoint and at worst the debugger gives up and throws an
	// exception at you.
#if  defined  DEBUG
	ldr 	r1, =HSX_PIT_GENERAL_REG
	ldr 	r0, =HSX_PIT_GENERAL_REG_DEFAULT_DEBUG
	str 	r0, [r1]

	ldr 	r1, =HSX_PIT_DATA_REG
	ldr 	r0, =HSX_PIT_DATA_REG_DEFAULT_DEBUG
	str 	r0, [r1]

	ldr 	r1, =HSX_PIT_INSTR_REG
	ldr 	r0, =HSX_PIT_INSTR_REG_DEFAULT_DEBUG
	str 	r0, [r1]
#else 
	ldr 	r1, =HSX_PIT_GENERAL_REG
	ldr 	r0, =HSX_PIT_GENERAL_REG_DEFAULT
	str 	r0, [r1]

	ldr 	r1, =HSX_PIT_DATA_REG
	ldr 	r0, =HSX_PIT_DATA_REG_DEFAULT
	str 	r0, [r1]

	ldr	r1, =HSX_PIT_INSTR_REG
	ldr 	r0, =HSX_PIT_INSTR_REG_DEFAULT
	str	r0, [r1]
#endif

	bx 	lr


/********************************************************************
 *  RemoveFlashShadow                                               
 *
 *      The first ROM bank is shadowed at address 0 upon reset. 
 *      This routine removes that shadow.                               
 *                                                                  
 ********************************************************************/
RemoveFlashShadow:
    ldr     r0, =RST_BASE
    str     r1, [r0, #RST_REMAP_OFF]
    bx      lr

/*************************************************************************
 *
 * InitializeStacks:
 *
 * Switch into each processor mode and initialize the stack pointer 
 * to a valid value.
 *
 *************************************************************************/
InitializeStacks:		 

	ldr      r2, =UBOOT_STACK_RAM_TOP
	/* These are needed by u-boot */
	sub	r2, r2, #CFG_MALLOC_LEN 	/* malloc area			    */
	sub	r2, r2, #CFG_GBL_DATA_SIZE 	/* bdinfo			    */

	/* confirm that we have allocated stack for all interrupts and exceptions */
#ifdef CONFIG_USE_IRQ
	sub	r2, r2, #(CONFIG_STACKSIZE_IRQ+CONFIG_STACKSIZE_FIQ)
#endif 
	sub	sp, r2, #12		/* leave 3 words for abort-stack    */
	ldr     r1, =BOUNDARY_STACK
	sub		r2, r2, #CONFIG_STACKSIZE
	str		r2, [r1]

	bx 	lr

/*************************************************************************
 *
 * InitializeSectons:
 *
 * Reclocate sections from ROM (LMA) to RAM (VMA). 
 *
 *************************************************************************/
InitializeSectons:
        /* First we must save the RAMSize variable so it is not destroyed by
	   this rountine */
	ldr     r12, =RAMSize
	ldr     r12, [r12]

	/* relocate sections to RAM (data, got, u_boot_cmd and bss) */
copy_sections:
	mov	r0, #ROM_BASE		 /* r0 <- current position of code   */
	ldr	r1, _relocate_dst_start  /* destination - is normally 0x0 */
	ldr	r2, _relocate_dst_end
2:	ldmia	r0!, {r3-r10}		/* copy from source address [r0]    */
	stmia	r1!, {r3-r10}		/* copy to   target address [r1]    */
	cmp	r1, r2			/* until source end addreee [r2]    */
	ble	2b

	/* copy interrupt vectors to ram location 0x0 */
copy_vectors:
	ldr 	r0, vector_start	/* r0 <- source addr of vectors */
	ldr	r2, vector_end 		/* r2 <- end of source address */
	ldr 	r1, =0x0		/* r1 <- dest addr of vectors */
1:	ldmia	r0!, {r3-r10}		/* copy from source address [r0]    */
	stmia	r1!, {r3-r10}		/* copy to   target address [r1]    */
	cmp	r0, r2			/* until source end addreee [r2]    */
	ble	1b

clear_bss:
	ldr	r0, _bss_start		/* find start of bss segment	    */
	ldr	r1, _bss_end		/* stop here			    */
	mov	r2, #0x0		/* clear			    */

clbss_l:
	str	r2, [r0]		/* clear loop...		    */
	add	r0, r0, #4
	cmp	r0, r1
	bne	clbss_l
	
        /* restore RAMSize */
	ldr r0, =RAMSize
	str r12, [r0]

	bx lr


/********************************************************************
 *  decarm_spin_begin, decarm_spin_end                              *
 *                                                                  *
 *  PARAMETERS:                                                     *
 *      None.                                                       *
 *                                                                  *
 *  DESCRIPTION:                                                    *
 *      This code segment writes ARM926 instructions.               *
 *                                                                  *
 *  RETURNS:                                                        *
 *      Never returns.                                              *
 ********************************************************************/
.global decarm_spin_begin
.global decarm_spin_end
decarm_spin_begin:
    ldr r0, [pc, #0xFF8]
    ldr r1, [pc, #0xFF8]
    cmp r0, #0 
    subeq pc, pc, #0x14
    cmp r1, #0
    subeq pc, pc, #0x1C
    bx r0
decarm_spin_end:

/*
 *************************************************************************
 *
 * Interrupt handling
 *
 *************************************************************************
 */
@
@ IRQ stack frame.
@
#define S_FRAME_SIZE	72

#define S_OLD_R0	68
#define S_PSR		64
#define S_PC		60
#define S_LR		56
#define S_SP		52

#define S_IP		48
#define S_FP		44
#define S_R10		40
#define S_R9		36
#define S_R8		32
#define S_R7		28
#define S_R6		24
#define S_R5		20
#define S_R4		16
#define S_R3		12
#define S_R2		8
#define S_R1		4
#define S_R0		0

#define MODE_SVC 0x13
#define I_BIT	 0x80

/*
 * use bad_save_user_regs for abort/prefetch/undef/swi ...
 * use irq_save_user_regs / irq_restore_user_regs for IRQ/FIQ handling
 */

	.macro	bad_save_user_regs
	sub	sp, sp, #S_FRAME_SIZE		@ carve out a frame on current user stack
	stmia	sp, {r0 - r12}			@ Save user registers (now in svc mode) r0-r12

	ldr     r2, =_armboot_start
	ldr 	r2, [r2]
	sub	r2, r2, #(CFG_MALLOC_LEN)
	sub	r2, r2, #(CFG_GBL_DATA_SIZE+8)	@ set base 2 words into abort stack
	ldmia	r2, {r2 - r3}			@ get values for "aborted" pc and cpsr (into parm regs)
	add	r0, sp, #S_FRAME_SIZE		@ grab pointer to old stack

	add	r5, sp, #S_SP
	mov	r1, lr
	stmia	r5, {r0 - r3}			@ save sp_SVC, lr_SVC, pc, cpsr
	mov	r0, sp				@ save current stack into r0 (param register)
	.endm

	.macro	irq_save_user_regs
	sub	sp, sp, #S_FRAME_SIZE
	stmia	sp, {r0 - r12}			@ Calling r0-r12
	add	r8, sp, #S_PC			@ !!!! R8 NEEDS to be saved !!!! a reserved stack spot would be good.
	stmdb	r8, {sp, lr}^			@ Calling SP, LR
	str	lr, [r8, #0]			@ Save calling PC
	mrs	r6, spsr
	str	r6, [r8, #4]			@ Save CPSR
	str	r0, [r8, #8]			@ Save OLD_R0
	mov	r0, sp
	.endm

	.macro	irq_restore_user_regs
	ldmia	sp, {r0 - lr}^			@ Calling r0 - lr
	mov	r0, r0
	ldr	lr, [sp, #S_PC]			@ Get PC
	add	sp, sp, #S_FRAME_SIZE
	subs	pc, lr, #4			@ return & move spsr_svc into cpsr
	.endm

	.macro get_bad_stack
	ldr     r13, =_armboot_start		@ setup our mode stack (enter in banked mode)
	ldr 	r13, [r13]
	sub	r13, r13, #(CFG_MALLOC_LEN)	@ move past malloc pool
	sub	r13, r13, #(CFG_GBL_DATA_SIZE+8) @ move to reserved a couple spots for abort stack

	str	lr, [r13]			@ save caller lr in position 0 of saved stack
	mrs	lr, spsr			@ get the spsr
	str	lr, [r13, #4]			@ save spsr in position 1 of saved stack

	mov	r13, #MODE_SVC			@ prepare SVC-Mode
	@ msr	spsr_c, r13
	msr	spsr, r13			@ switch modes, make sure moves will execute
	mov	lr, pc				@ capture return pc
	movs	pc, lr				@ jump to next instruction & switch modes.
	.endm

	.macro get_bad_stack_swi
	sub	r13, r13, #4			@ space on current stack for scratch reg.
	str	r0, [r13]			@ save R0's value.
	ldr     r0, =_armboot_start		@ get data regions start
	ldr 	r0, [r0]
	sub	r0, r0, #(CFG_MALLOC_LEN)	@ move past malloc pool
	sub	r0, r0, #(CFG_GBL_DATA_SIZE+8)	@ move past gbl and a couple spots for abort stack
	str	lr, [r0]			@ save caller lr in position 0 of saved stack
	mrs	r0, spsr			@ get the spsr
	str	lr, [r0, #4]			@ save spsr in position 1 of saved stack
	ldr	r0, [r13]			@ restore r0
	add	r13, r13, #4			@ pop stack entry
	.endm

	.macro get_irq_stack			@ setup IRQ stack
	ldr	sp, IRQ_STACK_START
	.endm

	.macro get_fiq_stack			@ setup FIQ stack
	ldr	sp, FIQ_STACK_START
	.endm

/*
 * exception handlers
 */
	.align	5
undefined_instruction:
	get_bad_stack
	bad_save_user_regs
	bl	do_undefined_instruction

	.align	5
software_interrupt:
	get_bad_stack_swi
	bad_save_user_regs
	bl	do_software_interrupt

	.align	5
prefetch_abort:
	get_bad_stack
	bad_save_user_regs
	bl	do_prefetch_abort

	.align	5
data_abort:
	get_bad_stack
	bad_save_user_regs
	bl	do_data_abort

	.align	5
not_used:
	get_bad_stack
	bad_save_user_regs
	bl	do_not_used

#ifdef CONFIG_USE_IRQ

	.align	5
irq:
	get_irq_stack
	irq_save_user_regs
	bl	do_irq
	irq_restore_user_regs

	.align	5
fiq:
	get_fiq_stack
	/* someone ought to write a more effiction fiq_save_user_regs */
	irq_save_user_regs
	bl	do_fiq
	irq_restore_user_regs

#else

	.align	5
irq:
	get_bad_stack
	bad_save_user_regs
	bl	do_irq

	.align	5
fiq:
	get_bad_stack
	bad_save_user_regs
	bl	do_fiq

#endif


/*
 *************************************************************************
 *
 * Strings used while initialization
 *
 *************************************************************************
 */
.data
.align 4
	.globl BOUNDARY_STACK
BOUNDARY_STACK:
	.word 0
