/****************************************************************************
 *
 *  Realtek 8201 Ethernet
 *
 *  Copyright (C) 2008 NXP semiconductors,Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, Version 2, as 
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but AS-IS and WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, TITLE, or 
 *  NONINFRINGEMENT.See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 */

#include "stbcfg.h"
#include "basetype.h"
#include "status_codes.h"

#include <common.h>
#include <command.h>
#include <net.h>
#include <malloc.h>
#include "rtl8201.h"

#ifdef CONFIG_RTL8201

#define MAX_EMAC 1
#define MAC_ADDRES_LEN 6

// AIX_3RDPARTY_CODE_CHANGE: js: look for MDIO support
#ifdef ETH_PHY_DISABLE_MDIO_SUPPORT
# define ETH_PHY_MDIO_SUPPORT 0
#else
# define ETH_PHY_MDIO_SUPPORT 1
#endif

extern CNXT_STATUS cnxt_gpio_set_output_level(u_int32 uGPIO, bool bOn);

#define EMAC_IRQ_ENABLES ( ENET_STAT_TXINT  | ENET_STAT_RXINT | ENET_STAT_ERR )
#define ENET_RXINFO_OWN             (0x80000000)    /* CPU/VMAC Ownership of buffer           */
#define ENET_TXINFO_OWN             (0x80000000)    /* CPU/VMAC Ownership of buffer           */
#define ENET_RXINFO_CPU_RX_LEN_MASK (0x000007FF)    /* Rx Length in this buffer to be Xmitted */
#define EMAC_FRAMESIZE_MIN 64
#define EMAC_FRAMESIZE_MAX 1520 /* set to this value to avoid buffer chaining.
                                   Actual frame size will be 1518 */
                                   
/* Tx FIFO threshold -start sending after 64 bytes are accumulated in FIFO, 
note that fifo depth is 512 bytes */

#define TX_FIFO_THRESHOLD 0xFFFFFC40

/* Tx Control Information Written By the CPU - Bit Masks */
#define ENET_TXINFO_CPU_TX_LEN      (0x000007FF)    /* Tx Length in this buffer to be Xmitted */
#define ENET_TXINFO_CPU_FIRST       (0x00010000)    /* First Tx buffer in the packet          */
#define ENET_TXINFO_CPU_LAST        (0x00020000)    /* Last Tx buffer in the packet           */

/* Tx Control Information Written By the VMAC - Bit Masks */

#define ENET_TXINFO_CARR_LOSS       (0x00200000)    /* Carrier Lost during xmission           */
#define ENET_TXINFO_DEFER           (0x00400000)    /* xmission deferred due to traffic       */
#define ENET_TXINFO_DROPPED         (0x00800000)    /* pkt dropped after 16 retries           */
#define ENET_TXINFO_LATE_COLL       (0x10000000)    /* Late Collision                         */
#define ENET_TXINFO_UFLO            (0x20000000)    /* Data not available on time             */

/*********************************************/
#define RTL8201CL_STATUS_REG         0x01
#define RTL8201CL_CTRL_RESET         0x8000
#define RTL8201CL_STATUS_LINK_UP     0x04
#define RTL8201CL_CTRL_REG           0x00
#define RTL8201CL_STATUS_COMPLETE    0x20          
#define RTL8201CL_CTRL_DUPLEX        0x0100

/*#define PHY_ADD     0x10 ; PHY_ADD is moved to 
  include/config/ird.h to make it configurable for all boards */

typedef unsigned char UINT8;
typedef unsigned short UINT16;
typedef unsigned long UINT32;

volatile UINT8 buf[2000];

/*Hardware Buffer Descriptor data structure*/
typedef struct
{
  UINT32 uInfo;  /* Info field of the Buffer Descriptor */
  UINT8 *puData; /* Pointer (to the data) field of the Buffer Descriptor */
} EMAC_BUF_DESCR;

/*Buffer contains Hardware buffer descriptor's Pointer field */
typedef struct
{
  void *puData;
} EMAC_LOCAL_BUF;

#define TxDescCount 1
#define RxDescCount 1

UINT8  rx_buf[4][1520];
UINT8  tx_buf[4][1520];
UINT8 txbuf_usemask;

/* extern */
extern UINT8 ReadCfgEepromC (int, int);

/* Static Buffer Dec Table */
#define bdt_size ((4 + 4) * 8)+ 16 /* Align to  8 bytes boundary */

static  UINT8 bdt[bdt_size];

typedef struct
{

  /*0 for eth0, 1 for eth1 */
  UINT16 uInterfaceNo;

  /* The MAC address of this port */
  UINT8  mac[6];

  /*Start of Memory block from where Tx and Rx BDT memory are allocated */
  UINT8  *puDescriptorBase;

  /*Points to the Buffer Descriptor in Tx BDT which will be used to store the
     upper layer supplied transmission buffer in the next Transmit call */
  EMAC_BUF_DESCR *tx_curr;

  /*Address of the First Buffer Descriptor in the Tx BDT */
  EMAC_BUF_DESCR *tx_first_desc;

  /*Address of the last Buffer Descriptor in the Tx BDT */
  EMAC_BUF_DESCR *tx_last_desc;

  /*Points to the Buffer Descriptor in Rx BDT from which we will start
     checking for BD which contains valid data */
  EMAC_BUF_DESCR *rx_curr;

  /*Address of the First Buffer Descriptor in the Rx BDT */
  EMAC_BUF_DESCR *rx_first_desc;

  /*Address of the Last Buffer Descriptor in the Rx BDT */
  EMAC_BUF_DESCR *rx_last_desc;

} CNXT_EMAC_DEVICE;

CNXT_EMAC_DEVICE gEmacDevice[MAX_EMAC];

#if ETH_PHY_MDIO_SUPPORT
UINT32  cnxt_mdio_phy_read(UINT32 reg);
void    cnxt_mdio_phy_write(UINT32 reg, UINT32 data);
UINT32  cnxt_mdio_wr_format(UINT32 reg, UINT32 data);
UINT32  cnxt_mdio_rd_format(UINT32 reg);
UINT32  cnxt_mdio_rd_data(void);
#endif

UINT32  CreateBufferDescriptorTable(CNXT_EMAC_DEVICE * pEmacDevice);
void    delay(void);

#ifdef CONFIG_CUSTOMER_ETH_ADDR
static int eth_addr_customer[ 6 ] = CONFIG_CUSTOMER_ETH_ADDR;
#endif

/******************************************************************************
FUNCTION:          delay                              
                                                                            
DESCRIPTION:     
  
INPUT PARAMETER    NONE

OUTPUT PARAMETER   NONE 

******************************************************************************/
void delay(void)
{
   volatile int i=0;
   for(i=0;i<500;i++);
}


/******************************************************************************
FUNCTION:            cnxt_initialize                        
                                                                            
DESCRIPTION:     
  
INPUT PARAMETER      bd_t * bis

OUTPUT PARAMETER     int 

*******************************************************************************/

int cnxt_initialize(bd_t * bis)
{
	struct eth_device *dev;
	static UINT8 addr[MAC_ADDRES_LEN];
   UINT8 eepromaddr[MAC_ADDRES_LEN];
	UINT16 retVal;
	char *tmp, *end;
	UINT16 i;
	UINT8 ethaddr[20];

	if (!(dev = (struct eth_device *) malloc (sizeof *dev))) 
	{
		printf("Failed to allocate memory\n");
		return -1;
	}
	memset(dev, 0, sizeof(*dev));

	#if I2C_CONFIG_EEPROM_ADDR != NOT_PRESENT
   /* query the emac address */
	for (i=0; i<MAC_ADDRES_LEN; i++)
   {
      eepromaddr[i]=   ReadCfgEepromC(1, (BOARD_CONFIG_MAC_ADDR_OFFSET+i));
   }
   #else
   /* use the default EMAC address defined in rtl8201.h */
	for (i=0; i<MAC_ADDRES_LEN; i++)
   {
      eepromaddr[i]= EMAC0_ADDR[i];
   }
   #endif

	tmp = getenv ("ethaddr"); 
	if (NULL == tmp)
	{

#ifdef CONFIG_CUSTOMER_ETH_ADDR
    if(  eepromaddr[0] != eth_addr_customer[0] 
      || eepromaddr[1] != eth_addr_customer[1] 
      || eepromaddr[2] != eth_addr_customer[2] 
      )
      for( i = 0; i < MAC_ADDR_LEN; i ++ )
        eepromaddr[i] = eth_addr_customer[i];
#endif
      /* set the ethaddr env variable */
		sprintf ((char*) ethaddr, "%02X:%02X:%02X:%02X:%02X:%02X",
				 eepromaddr[0], eepromaddr[1],
				 eepromaddr[2], eepromaddr[3],
				 eepromaddr[4], eepromaddr[5]) ;
		setenv ("ethaddr", (char*) ethaddr);
      for(i=0;i<MAC_ADDRES_LEN;i++)
      {
         addr[i] = eepromaddr[i];
      }

	}
   else
   {
      for (i=0; i<MAC_ADDRES_LEN; i++) 
      {
       	addr[i] = tmp ? simple_strtoul(tmp, &end, 16) : 0;
       	if (tmp)
       		tmp = (*end) ? end+1 : end;
      }
      /* compare the ethaddr with the emac address stored in EEPROM,
       * if they are different, set the ethaddr to the ones from EEPROM 
       */
	   #if I2C_CONFIG_EEPROM_ADDR != NOT_PRESENT
      if( (memcmp( addr, eepromaddr, MAC_ADDRES_LEN)) )
      {
#ifdef CONFIG_CUSTOMER_ETH_ADDR
         if(  eepromaddr[0] != eth_addr_customer[0] 
           || eepromaddr[1] != eth_addr_customer[1] 
           || eepromaddr[2] != eth_addr_customer[2] 
           )
           for( i = 0; i < MAC_ADDR_LEN; i ++ )
              eepromaddr[i] = eth_addr_customer[i];
#endif
         /* different, set the ethaddr to be the one from eeprom*/
         sprintf ((char*) ethaddr, "%02X:%02X:%02X:%02X:%02X:%02X",
                eepromaddr[0], eepromaddr[1],
                eepromaddr[2], eepromaddr[3],
                eepromaddr[4], eepromaddr[5]) ;
         setenv ("ethaddr", (char*) ethaddr);
         for(i=0;i<MAC_ADDRES_LEN;i++)
         {
            addr[i] = eepromaddr[i];
         }
      }
      #endif
  }

   /* save the EMAC address to struct dev */
   for(i=0;i<MAC_ADDRES_LEN;i++)
   {
      dev->enetaddr[i] = addr[i];
   }

   retVal = rtl8201_initialize(dev,bis);       

	sprintf(dev->name, "rtl8201");
	dev->init = rtl8201_init;
	dev->halt = rtl8201_halt;
	dev->send = rtl8201_tx;
	dev->recv = rtl8201_rx;

	eth_register(dev);

	return retVal;
}

/******************************************************************************
FUNCTION:            CreateBufferDescriptorTable                        
                                                                            
DESCRIPTION:     
  
INPUT PARAMETER      CNXT_EMAC_DEVICE * pEmacDevice

OUTPUT PARAMETER     NONE 

*******************************************************************************
*/
UINT32  CreateBufferDescriptorTable(
               CNXT_EMAC_DEVICE * pEmacDevice)
{
  UINT32 uTmpAddr;
  UINT16 uCount;

  EMAC_BUF_DESCR *pBufDescr;

  pEmacDevice->puDescriptorBase = bdt;
  if ( (void *)0 != pEmacDevice->puDescriptorBase)
  {
      /*TX Buffer Descriptor Table Information */

      /* First round up base pointer */
      uTmpAddr =
      (UINT32) (pEmacDevice->puDescriptorBase);
      uTmpAddr += 0x7;
      uTmpAddr &= ~(0x7);

      pEmacDevice->tx_first_desc = (EMAC_BUF_DESCR *) uTmpAddr;

      pEmacDevice->tx_last_desc =  pEmacDevice->tx_first_desc + (TxDescCount - 1);

      /*TX BDT ptr field initialization */
      pBufDescr = pEmacDevice->tx_first_desc;
      for (uCount = 0; uCount < TxDescCount; uCount++)
      {
         /* Prepare the Buffer descriptors */
         pBufDescr->puData = (void *)0;
         /*Imp: CRC is hardware supported, Network stack should exclude CRC */
         pBufDescr->uInfo =
         ENET_TXINFO_CPU_FIRST | ENET_TXINFO_CPU_LAST;
         pBufDescr++;
      }
      pEmacDevice->tx_curr = pEmacDevice->tx_first_desc;

      /* First round up base pointer */
      uTmpAddr =  (UINT32) (pEmacDevice->tx_last_desc +  1);
      uTmpAddr += 0x7;
      uTmpAddr &= ~(0x7);
      pEmacDevice->rx_first_desc = (EMAC_BUF_DESCR *) uTmpAddr;

      pEmacDevice->rx_last_desc =  pEmacDevice->rx_first_desc + (RxDescCount - 1);

      /*RX BDT ptr field initialization */
      pBufDescr = pEmacDevice->rx_first_desc;

      for (uCount = 0; uCount < RxDescCount; uCount++)
      {
         pBufDescr->puData = rx_buf[uCount];
         pBufDescr->uInfo = ENET_RXINFO_OWN | 0x5f0;

         pBufDescr++;
      }
      pEmacDevice->rx_curr = pEmacDevice->rx_first_desc;

  }
  return 0;
}

#if ETH_PHY_MDIO_SUPPORT

/******************************************************************************
FUNCTION:        cnxt_mdio_rd_data                               
                                                                          
DESCRIPTION:     Read MDIO access regsiter and mask first 2 byte to take data
  
INPUT PARAMETER  NONE

OUTPUT PARAMETER NONE 

*******************************************************************************
*/
UINT32 cnxt_mdio_rd_data(void)
{
    UINT32 data;
    data = CNXT_GET(ENET_MDIO_REG(0),0xffffffff); 
    data = data & 0x0000FFFF;
    return data;
}

/******************************************************************************
FUNCTION:          cnxt_mdio_rd_format                          
                                                                            
DESCRIPTION:       Function used to perform mdio read opration
                   set operation Code to read , enable start of frame 
                   delimeter     
  
INPUT PARAMETER    UINT32 reg

OUTPUT PARAMETER   NONE 

*******************************************************************************
*/
UINT32  cnxt_mdio_rd_format(UINT32 reg)
{
    
    return((1 << 30) | (1 << 29) | (PHY_ADD << 23) | (reg << 18) | (1 << 17)); 

}

/******************************************************************************
FUNCTION:           cnxt_mdio_wr_format                              
                                                                            
DESCRIPTION:        Function used to perform mdio write opration
                    set operation Code to write,enable start of frame 
                    delimeter. 
  
INPUT PARAMETER     UINT32 reg
                    UINT32 data   

OUTPUT PARAMETER    NONE 

*******************************************************************************
*/
UINT32 cnxt_mdio_wr_format(UINT32 reg, UINT32 data)
{

    return ((1 << 30) | (1 << 28) | (PHY_ADD << 23) | (reg << 18) | (1 << 17) | data); 
}

/******************************************************************************
  FUNCTION:            cnxt_mdio_phy_write                              
                                                                            
  DESCRIPTION:     
  
INPUT PARAMETER    UINT32 reg
                                UINT32 data

OUTPUT PARAMETER NONE 

*******************************************************************************
*/
void cnxt_mdio_phy_write(UINT32 reg, UINT32 data)
{
    UINT32 rd_data;

    /* Clearing MDIO Rd/Wr complete bit     */
    
    CNXT_SET(ENET_STAT_REG(0),0xffffffff,0x00001000);
    
    CNXT_SET(ENET_MDIO_REG(0), 0xffffffff,cnxt_mdio_wr_format(reg,data));
    
    /*Wait till MDIO Rd is complete since MDIO CLK is slower than
     system clock*/
    
    while(1)
    {
        rd_data = CNXT_GET(ENET_STAT_REG(0),0xffffffff);
        rd_data = rd_data & 0x00001000; 
        if(rd_data)     
        {
            break;
        }
    }
    
    /* Clearing MDIO Rd/Wr complete bit */      
    CNXT_SET(ENET_STAT_REG(0),0xffffffff , rd_data);
    
}

/******************************************************************************
FUNCTION:               cnxt_mdio_phy_read 

DESCRIPTION:            Set  MDIO access register to accept input mode and 
                        Read the Ethernet status regsiter
  
INPUT PARAMETER         UINT32 reg

OUTPUT PARAMETER        NONE 

*******************************************************************************
*/
UINT32 cnxt_mdio_phy_read(UINT32 reg)
{
    UINT32 rd_data;

    /* Clearing MDIO Rd/Wr complete bit     */
    CNXT_SET(ENET_STAT_REG(0),0xffffffff,0x00001000);
    CNXT_SET(ENET_MDIO_REG(0), 0xffffffff,cnxt_mdio_rd_format(reg));
    
    /*Wait till MDIO Rd is complete since MDIO CLK is slower than
     system clock*/ 
    while(1)
    {
        rd_data = CNXT_GET(ENET_STAT_REG(0),0xffffffff);
        rd_data = rd_data & 0x00001000; 
        if(rd_data)     
        {
            break;
        }
    }
    
    /* Clearing MDIO Rd/Wr complete bit */      
    CNXT_SET(ENET_STAT_REG(0),0xffffffff , rd_data);
    rd_data = cnxt_mdio_rd_data();
    
    return rd_data;     

    
}

#endif

/******************************************************************************
FUNCTION:        rtl8201_initialize                         
                                                                            
DESCRIPTION:     Initializes VMAC controller.
                 reset the phy by clearing GPIO pin , read the phy status to to
                 ensure that link has up or not.Set PHY and EMAC cntroler for 
                 Full Duplex mode  full duplex mode of communication.
                 Set promiscuous mode to accept all pkts , store input MAC adreess 
                 and enable all Interrupts.Create FIFO buffers fro tx and rx , 
                 Change the default tx fifo threshold to 64.                          
                                
INPUT PARAMETER  struct eth_device *dev
                 bd_t * bd

OUTPUT PARAMETER NONE 

*******************************************************************************
*/
int rtl8201_initialize(struct eth_device *dev,bd_t * bd)
{
  UINT32 data, val, tmpVal ;
  CNXT_EMAC_DEVICE *pEmacDevice;
  pEmacDevice = &gEmacDevice[0];
  UINT8 mac_1[6];
  UINT16 i;
 
  for(i=0;i<MAC_ADDRES_LEN;i++)
  {
	  mac_1[i] = dev->enetaddr[i];
  }
    
#if ETH_PHY_MDIO_SUPPORT
  /* check phy reset complete */
  while(1)
  {
     data = cnxt_mdio_phy_read(RTL8201CL_CTRL_REG);
     if((data & RTL8201CL_CTRL_RESET) == 0)    
     { 
         break;
     }

  }
#endif

  /*Reset the ENABLE register. i.e. Disable all interrupts */
  CNXT_SET((ENET_ENABLE_REG(0)),0xffffffff, 0);

  /* Wait to complete the operation */
  delay();

  /* Set number of TxBDs and RxBDs to be 1. */
  val = CNXT_GET(ENET_CONTROL_REG(0) ,0xffffffff);
  CNXT_SET(ENET_CONTROL_REG(0) ,0xffffffff , 
                   (val  | (1 << ENET_CONTROL_TXBDTLEN_SHIFT) | (1 << ENET_CONTROL_RXBDTLEN_SHIFT)));
  delay();

   /*Write the Ethernet MAC address to ADDR registers (ADDRL, ADDRH). */
   tmpVal = ((mac_1[3] << 24)
                | (mac_1[2] << 16)
                | (mac_1[1] << 8) | (mac_1[0] << 0));

   CNXT_SET((ENET_ADDRL_REG(0)), 0xffffffff,tmpVal);
   
   tmpVal =( (mac_1[5] << 8) | (mac_1[4] << 0));
   CNXT_SET((ENET_ADDRH_REG(0)), 0x0000ffff,tmpVal);

  /*Clear (set 0) hash filter values (to accept or reject the 64 group of
         logical address) to LAF  registers (LAFL, LAFH). */
  CNXT_SET((ENET_LAFL_REG(0)), 0xffffffff,0);
  CNXT_SET((ENET_LAFH_REG(0)),0xffffffff, 0);
  delay();

  CreateBufferDescriptorTable(pEmacDevice);
  delay();
  
  /*Write address of the start of transmit buffer descriptor table to TXRINGPTR register. */
  tmpVal = (unsigned long) pEmacDevice->tx_first_desc;
  CNXT_SET ((ENET_TXRINGPTR_REG(0)), 0xffffffff, tmpVal);
  /*Write address of the start of recieve buffer descriptor table to TXRINGPTR register. */
  tmpVal = (unsigned long) pEmacDevice->rx_first_desc;
  CNXT_SET((ENET_RXRINGPTR_REG(0)),0xffffffff, tmpVal);

  /*Write the poll rate value to POOLRATE register. */
  CNXT_SET((ENET_POLLRATE_REG(0)),0xffffffff, ENET_POLLRATE_MASK);

  /*Clear all Interrupt Status bits */
  CNXT_SET((ENET_STAT_REG(0)),0xffffffff, 0x0000171F);
  /*Enable all Interrupts */
  CNXT_SET((ENET_ENABLE_REG(0)),0xffffffff, EMAC_IRQ_ENABLES);
  delay();
  
  val = CNXT_GET(ENET_CONTROL_REG(0) ,0xffffffff);
  val |= (ENET_CONTROL_ENABLE | ENET_CONTROL_TXRUN | ENET_CONTROL_RXRUN);
  /* And set all above */   
  CNXT_SET((ENET_CONTROL_REG(0)), 0xffffffff, val);
  delay();

  (*((volatile unsigned long *)ENET_XTRACTRL_REG(0))) = 0;
  delay();
  /* Set Tx Auto PAD , Tx and Rx 8 word burst mode .. */

  tmpVal = (*((volatile unsigned long *)ENET_XTRACTRL_REG(0))) & (~ENET_XTRACTRL_RESET_MASK);

  (*((volatile unsigned long *)ENET_XTRACTRL_REG(0))) = tmpVal;
  /* Change the default tx fifo threshold to 64 */
  tmpVal = ((*((volatile unsigned long *)ENET_XTRACTRL_REG(0))) | (ENET_XTRACTRL_TXTHRESH_MASK)) &
      TX_FIFO_THRESHOLD;
  (*((volatile unsigned long *)ENET_XTRACTRL_REG(0))) = tmpVal;      
  delay();
	  
  tmpVal = (*((volatile unsigned long *)ENET_XTRACTRL_REG(0))) ;
  tmpVal |= ENET_XTRACTRL_BVCIFIX_MASK
              | ENET_XTRACTRL_TXAUTOPAD_YES
              | ENET_XTRACTRL_RXBURST8
              | ENET_XTRACTRL_TXBURST8
              | ENET_XTRACTRL_XTRAENA | ENET_XTRACTRL_LOCKFIX_ENA;
  (*((volatile unsigned long *)ENET_XTRACTRL_REG(0))) = tmpVal;
  delay();

  return 0;
            
}

/******************************************************************************
FUNCTION:        rtl8201_init                         
                                                                            
DESCRIPTION:     Dummy

INPUT PARAMETER  struct eth_device *dev
                 bd_t * bd

OUTPUT PARAMETER NONE 

*******************************************************************************
*/
int rtl8201_init(struct eth_device *dev,bd_t * bd)
{
   UINT32 val;

#if ETH_PHY_MDIO_SUPPORT
   UINT32 data, temp=0;

  /*Wait for some time to check the Link is up or not */ 
  while(temp<20000)
  {
        data = cnxt_mdio_phy_read(RTL8201CL_STATUS_REG);
             temp++;
        if ((data & RTL8201CL_STATUS_LINK_UP)) 
        {
            puts("Ethernet Link Up\n");
                    temp=0;
            break;
        }
  }

  /* If Link is not up just  throw an error  */
  if(temp == 20000)
  {
        puts("Link is down - Please ensure that Ethernet cable is inserted \n" );
        return -1;
  }

  delay();
   
  /* If auto-negotiation process is completed , do default
     operation, full duplex and 100mbps  */
  data = cnxt_mdio_phy_read(RTL8201CL_STATUS_REG);
  if ( (data & RTL8201CL_STATUS_COMPLETE ) == 0 )
  {
     data = cnxt_mdio_phy_read(RTL8201CL_CTRL_REG );
     data = data | 0x3300; 
     cnxt_mdio_phy_write(RTL8201CL_CTRL_REG,data);
        
     /* Wait for Auto-negotiation to complete */
     while(1)
     {
        data = cnxt_mdio_phy_read(RTL8201CL_STATUS_REG);
                     
        if ((data & RTL8201CL_STATUS_COMPLETE) != 0)
        {
              break;
        }
     }        
  }
  
  data = cnxt_mdio_phy_read(RTL8201CL_CTRL_REG);
  if ((data & RTL8201CL_CTRL_DUPLEX) != 0)
  {
     /* Set the full duplex mode of communication */
     val = CNXT_GET(ENET_CONTROL_REG(0) ,0xffffffff);
     CNXT_SET(ENET_CONTROL_REG(0) ,0xffffffff, (val | ENET_CONTROL_ENBFULL)); /* full duplex get from PHY */
  }
#else
  val = CNXT_GET(ENET_CONTROL_REG(0) ,0xffffffff);
  CNXT_SET(ENET_CONTROL_REG(0) ,0xffffffff, (val | ENET_CONTROL_ENBFULL)); /* full duplex get from PHY */
#endif
  /* Disable promiscuous mode */
  val = CNXT_GET(ENET_CONTROL_REG(0) ,0xffffffff);
  CNXT_SET(ENET_CONTROL_REG(0) , 0xffffffff, (val & ~(ENET_CONTROL_PROM)));    
  
  return 0;

}

/******************************************************************************
FUNCTION:        rtl8201_halt                        
                                                                            
DESCRIPTION:     Dummy                   
                                
INPUT PARAMETER  struct eth_device *dev

OUTPUT PARAMETER NONE 

*******************************************************************************/
void rtl8201_halt (struct eth_device *dev)
{
}


/******************************************************************************
FUNCTION:          rtl8201_rx                               
                                                                            
DESCRIPTION:       This function get buffers from PHY and copy those to out 
                   parametr buf. First reads the staus of EMAC ,whether it 
                   receives rx interrupt or not. If there is rx interrupt then 
                   it Disable Rx Interrupt till the completion of the Interrupt
                   processing , copy the receiving frame from rx FIFO to given 
                   buffer.Enable Rx interrupt again to catch further data.
  
INPUT PARAMETER    NONE

OUTPUT PARAMETER   INT 

*******************************************************************************
*/
int  rtl8201_rx(struct eth_device *dev)
{
   UINT32 uIrq;
   register UINT32 info;
   register UINT8  *ptr;
   UINT32 uLen;
   UINT32 uStatus;
   UINT32 uInterfaceNo =0;
   CNXT_EMAC_DEVICE *pEmacDevice;
   EMAC_BUF_DESCR *rx_desc_ptr;
   EMAC_BUF_DESCR *pBufDescr;
   pEmacDevice = &gEmacDevice[uInterfaceNo];
   pBufDescr = pEmacDevice->rx_first_desc;

   uLen = 0;

    /*Read the interrupt status register */
   uIrq =  (read_reg(ENET_STAT_REG(uInterfaceNo)))  & ~(ENET_STAT_MDIO);
   /*Clear the interrupt */
   CNXT_SET(ENET_STAT_REG(uInterfaceNo),0xffffffff,  uIrq);

   if (ENET_STAT_RXINT & uIrq)
   {

      /* Disable Rx Interrupt till the completion of the Interrupt processing */
      uStatus = (read_reg(ENET_ENABLE_REG(uInterfaceNo)))  & (~ENET_STAT_RXINT);
      write_reg(ENET_ENABLE_REG(uInterfaceNo), uStatus);

      rx_desc_ptr = pEmacDevice->rx_curr;

      info = rx_desc_ptr->uInfo;
      ptr = rx_desc_ptr->puData;

      if((!(info & ENET_RXINFO_OWN)))
      {
         /* Received a packet */
         uLen = info & ENET_RXINFO_CPU_RX_LEN_MASK;
            if (uLen >= EMAC_FRAMESIZE_MIN)
            {
              uLen -= 4;
            }
            else
            {
               puts("Error:Short Frame\n");

            }

	     /*Copy the received buffer */
            memcpy(&buf ,(UINT8 *)ptr, uLen);

	     /* Pass the packet to the higher layer */
            NetReceive((unsigned char*) &buf ,uLen);

            rx_desc_ptr->uInfo = (EMAC_FRAMESIZE_MAX) | ENET_RXINFO_OWN;
            rx_desc_ptr++;
            if(rx_desc_ptr > pEmacDevice->rx_last_desc )
               pEmacDevice->rx_curr =  pEmacDevice->rx_first_desc;
            else
               pEmacDevice->rx_curr =  rx_desc_ptr;
      }
      uStatus = (read_reg(ENET_ENABLE_REG(uInterfaceNo)))  | ENET_STAT_RXINT;
      write_reg(ENET_ENABLE_REG(uInterfaceNo),uStatus);
   }
   else if(ENET_STAT_ERR & uIrq)
   {
       puts("Error in interrupt\n");
       if (ENET_STAT_TXCH & uIrq)
       {
         puts("Error: Tx chain err\n");
       }
      if (ENET_STAT_MISSERR & uIrq)
      {
          puts("Error: Miss \n");
      }

      if (ENET_STAT_RXCRC & uIrq)
      {
          puts("Error: RX crc\n");
      }

      if (ENET_STAT_RXFRAME & uIrq)
      {
          puts("Error: RX frame\n");
      }

      if (ENET_STAT_RXOFLOW & uIrq)
      {
          puts("Error: RX Overflow\n");
      }
   }
   
   return (UINT16)uLen;
}

/******************************************************************************
FUNCTION:          rtl8201_tx                             
                                                                            
DESCRIPTION:       This function send buffers to  PHY. First reads the staus of 
                   EMAC controller,whether it receives tx interrupt or not.
                   if there is tx interrupt then it Disable tx Interrupt till the 
                   completion of the Interrupt processing ,copy the frame from 
                   buf to  tx FIFO .
                   Enable Tx interrupt again to for processing  further data.
  
INPUT PARAMETER    UINT8 *buf
                   UINT32 len 

OUTPUT PARAMETER   NONE 

*******************************************************************************
*/

int rtl8201_tx (struct eth_device *dev, volatile void *buf, int len)
{

   EMAC_BUF_DESCR *tx_desc_ptr;
   CNXT_EMAC_DEVICE *pEmacDevice;
   UINT32 uInterfaceNo =0;
   UINT32 uIrq;
   UINT32 uLenBuf;
   register UINT32 info;
   register UINT8 *ptr;
   UINT32 uDone;
   UINT32 uStatus;
   UINT32 uPktSent = 0;

   pEmacDevice = &gEmacDevice[uInterfaceNo];
   tx_desc_ptr = pEmacDevice->tx_curr;

   if (!(tx_desc_ptr->uInfo & ENET_TXINFO_OWN))
   {
       /* Is the BDT processed for last Transmission */
     if ((void *)0 == tx_desc_ptr->puData)
     {
         uDone = 1; /* Updated BDT */
         tx_desc_ptr->puData =  (UINT8*) buf;

         tx_desc_ptr->uInfo = ENET_TXINFO_CPU_FIRST |
         ENET_TXINFO_CPU_LAST | ENET_TXINFO_OWN | len ;
     }
   }
   
    if (1 == uDone)
    {
       pEmacDevice->tx_curr = tx_desc_ptr;
       write_reg(ENET_STAT_REG(uInterfaceNo), ENET_STAT_TXPOLL);
    }

    delay();
    tx_desc_ptr = pEmacDevice->tx_curr;
    /* read the interrupt status register */
    uIrq =  (read_reg(ENET_STAT_REG(uInterfaceNo)))  & ~(ENET_STAT_MDIO);

    if (ENET_STAT_TXINT  & uIrq)
    {

       /* Disable Tx Interrupt till the completion of the Interrupt processing */
       uStatus = (read_reg(ENET_ENABLE_REG(uInterfaceNo))) & (~ENET_STAT_TXINT );
       write_reg(ENET_ENABLE_REG(uInterfaceNo), uStatus);

       info = tx_desc_ptr->uInfo;
       ptr = tx_desc_ptr->puData;

       /* BDT owned by CPU and it is used for transmission */
       while ((!(info & ENET_TXINFO_OWN)) && ptr != (void *)0)
       {

           uLenBuf = info & ENET_TXINFO_CPU_TX_LEN;
           uPktSent = 1;

           if(ENET_TXINFO_CARR_LOSS == (ENET_TXINFO_CARR_LOSS & info))
           {
             puts("Warning: Carrier sense lost\n");
              uPktSent = 0;
            }

           if(ENET_TXINFO_DEFER == (ENET_TXINFO_DEFER & info))
           {
               puts("Warning: Tx Deferred\n");
           }


           if(ENET_TXINFO_DROPPED == (ENET_TXINFO_DROPPED & info))
           {
             puts("Error: Packet dropped\n");
              uPktSent = 0;
            }


           if(ENET_TXINFO_LATE_COLL == (ENET_TXINFO_LATE_COLL & info))
           {
             puts("Error: Late collision\n");
              uPktSent = 0;
           }

           if(ENET_TXINFO_UFLO == (ENET_TXINFO_UFLO & info))
           {
             puts("Error: Tx underflow\n");
              uPktSent = 0;
            }

           if(1 == uPktSent)
           {
              tx_desc_ptr->puData = (void *)0;
           }
           /* Clear the descriptor */

           tx_desc_ptr++;

           /* Loopback to orginal location for circular buffer implementation */
           if(tx_desc_ptr > pEmacDevice->tx_last_desc)
           {
              tx_desc_ptr = pEmacDevice->tx_first_desc;
           }

           info = tx_desc_ptr->uInfo;
           ptr = tx_desc_ptr->puData;

       }
       pEmacDevice->tx_curr = tx_desc_ptr ;
       /* enable the Tx Interrupt */
       uStatus = (read_reg(ENET_ENABLE_REG(uInterfaceNo))) | ENET_STAT_TXINT ;
       write_reg(ENET_ENABLE_REG(uInterfaceNo), uStatus);
       if(1 != uPktSent)
         return -1;
       else return 0;
    }
    
   return 0;
}

/******************************************************************************
FUNCTION:            cnxt_terminate                        
                                                                            
DESCRIPTION:     
  
INPUT PARAMETER      

OUTPUT PARAMETER      

*******************************************************************************/

void cnxt_terminate()
{
  UINT32 uCount, uTmpVal;
  UINT32 uInterfaceNo =0;

  printf("Conexant Uboot Ethernet driver termination\n");

  /*Stop the EMAC operation */
  uTmpVal = (*((volatile unsigned long *)ENET_CONTROL_REG(uInterfaceNo)));
  uTmpVal &= (~(ENET_CONTROL_ENABLE | ENET_CONTROL_TXRUN | ENET_CONTROL_RXRUN));
  (*((volatile unsigned long *)ENET_CONTROL_REG(uInterfaceNo))) = uTmpVal;

  /*Reset the ENABLE register. i.e. Disable all interrupts */
  CNXT_SET((ENET_ENABLE_REG(uInterfaceNo)),0xffffffff, 0);


  /*Disable the EMAC by writing 0 to Control Register */
  (*((volatile unsigned long *)ENET_CONTROL_REG(uInterfaceNo))) = 0;
  (*((volatile unsigned long *)ENET_LAFL_REG(uInterfaceNo))) = 0;
  (*((volatile unsigned long *)ENET_LAFH_REG(uInterfaceNo))) = 0;
  (*((volatile unsigned long *)ENET_TXRINGPTR_REG(uInterfaceNo))) = 0;
  (*((volatile unsigned long *)ENET_RXRINGPTR_REG(uInterfaceNo))) = 0;

  /*Wait till 0 is written into Ring pointer registers */
  uCount = 0;
  do
  {
      uCount++;
      uTmpVal =  (*((volatile unsigned long *)ENET_TXPTRREAD_REG(uInterfaceNo)));
      uTmpVal |= (*((volatile unsigned long *)ENET_RXPTRREAD_REG(uInterfaceNo)));
      if (uCount > EMAC_LOCKUP_WAIT)
      {
           break;
      }
  }
  while (uTmpVal != 0);

}

#endif /* CONFIG_RTL8201 */

