/*
 * COM1 NS16550 support
 * originally from linux source (arch/ppc/boot/ns16550.c)
 * modified to use CFG_ISA_MEM and new defines
 */
 
#include <config.h>

#ifdef CFG_NS16550

#include <ns16550.h>

#define LCRVAL LCR_8N1					/* 8 data, 1 stop, no parity */
#define MCRVAL (MCR_DTR | MCR_RTS)			/* RTS/DTR */
//#define FCRVAL (FCR_FIFO_EN_128 | FCR_RXSR | FCR_TXSR)	/* Clear & enable FIFOs */
#define FCRVAL FCR_CNXT

//AIX_3RDPARTY_CODE_CHANGED
int NS16550_input_disabled = 0;

void NS16550_init (NS16550_t com_port, int baud_divisor)
{
  // AIX_3RDPARTY_CODE_CHANGE: js: CX2450x needs the divisor-1 (see datasheet), 
  baud_divisor--; // do not change calculation functio! (it is shared code)

#ifdef CONFIG_OMAP
	com_port->mdr1 = 0x7;	/* mode select reset TL16C750*/
#endif
	com_port->lcr = LCR_BKSE | LCRVAL;
	com_port->exp = 0;
	com_port->dll = baud_divisor & 0xff;
	com_port->dlm = (baud_divisor >> 8) & 0xff;
	com_port->lcr = LCRVAL;
	com_port->ier = 0x00;
	com_port->mcr = MCRVAL;
	com_port->fcr = FCRVAL;

#if defined(CONFIG_OMAP)
#if defined(CONFIG_APTIX)
	com_port->mdr1 = 3;	/* /13 mode so Aptix 6MHz can hit 115200 */
#else
	com_port->mdr1 = 0;	/* /16 is proper to hit 115200 with 48MHz */
#endif
#endif
}

void NS16550_reinit (NS16550_t com_port, int baud_divisor)
{
	NS16550_init (com_port, baud_divisor);
}

void NS16550_putc (NS16550_t com_port, char c)
{
	while ((com_port->lsr & LSR_THRE) == 0) {};
        com_port->thr = c;
}

char NS16550_getc (NS16550_t com_port)
{
	while ((com_port->lsr & LSR_DR) == 0) {
#ifdef CONFIG_USB_TTY
		extern void usbtty_poll(void);
		usbtty_poll();
#endif
	}
// AIX_3RDPARTY_CODE_CHANGED: js: disable serial input capability
  if( NS16550_input_disabled )
    return 0;
  return (com_port->rbr);
}

int NS16550_tstc (NS16550_t com_port)
{
  if( NS16550_input_disabled )
    return 0;
  return ((com_port->lsr & LSR_DR) != 0);
}

#endif
