#ifndef __CONFIG_H
#define __CONFIG_H

/*-----------------------------------------------------------------------
 * High Level Configuration Options
 */
#define CONFIG_ARM1176        1  /* This is an arm1167 CPU core */
#define CONFIG_IRD            1  /* NXP IRD */
#define CONFIG_RTL8201	      1  /* Ethernet RTL8201 Driver */
#define CONFIG_TLV_IMAGE      1  /* needed for booting with NXP tlv images */
/*#define DEBUG               1  */

/* Memory configuration on board */
#include <asm/sizes.h>

/* Clock config to target*/
#define V_SCLK                54000000    /* 54MHz by default */
/*#define CFG_CPU_SPEED       666666666*/ /* 666MHz */

/* input clock of PLL */
#define CONFIG_SYS_CLK_FREQ   V_SCLK

/*-----------------------------------------------------------------------
 * Flash & Environment
 */
#undef   CFG_NO_FLASH

/* environment start address offset  */
#define CONFIG_ENV_OFFSET     (0x00060000)

// AIX_3RDPARTY_CODE_CHANGED: much changes in this file...
#ifdef CONFIG_USE_NFC   /* defined in include/config.h */

# undef CONFIG_ENV_IS_IN_FLASH
# define CONFIG_ENV_IS_IN_NFC
# define CFG_NO_FLASH
# define CONFIG_CMD_NFC

/* the env variables start address */
# define CFG_ENV_ADDR          (CONFIG_ENV_OFFSET)

#elif defined(CONFIG_USE_NOR) /* defined in include/config.h */

# undef CFG_NO_FLASH
# define CONFIG_ENV_IS_IN_FLASH

/* use CFI flash driver */
# define CONFIG_FLASH_CFI_DRIVER
# define CFG_FLASH_CFI
/* try to undef first to verify, normally doesn't need it */
# define CFG_FLASH_CFI_AMD_RESET // aixsolve:js: try INTEL reset breaks the AMD flash reset for ST M29W128
# define CFG_FLASH_CFI_WIDTH   (FLASH_CFI_16BIT)

# define CFG_FLASH_BASE        (0xF0000000)
# define CFG_FLASH_BANKS_LIST  {CFG_FLASH_BASE, CFG_FLASH_BASE+flash_info[0].size}
# define CFG_FLASH_QUIET_TEST

/* Use buffered writes (~10x faster) */
# define CFG_FLASH_USE_BUFFER_WRITE  1

/* Use hardware sector protection, need to enable it */
# define CFG_FLASH_PROTECTION        1

/* print 'E' for empty sector on flinfo */
# define CFG_FLASH_EMPTY_INFO        1

/* max number of flash banks */
# define CFG_MAX_FLASH_BANKS         2

# define CFG_MAX_FLASH_SECT          1024
# define CFG_MAX_FLASH_BANKS_DETECT  2

/* the env variables start address */
# define CONFIG_ENV_ADDR             (CFG_FLASH_BASE + CONFIG_ENV_OFFSET)
#endif

#define CONFIG_ENV_OVERWRITE              /* allow to overwrite serial and ethaddr */
#define CONFIG_ENV_SIZE             SZ_128K     /* Total Size of Environment Sector, has to be multiple of sectior size */
#define CFG_MONITOR_BASE            CFG_FLASH_BASE
#define CFG_MONITOR_LEN             SZ_256K     /* Reserve 256kB for u-boot */
#define CONFIG_EXTRA_ENV_SETTINGS   "serverip=18.0.0.253          \0" \
                                    "ipaddr=18.0.0.40             \0" \
                                    "kernel_rom_addr=0xF0100000   \0" \
                                    "loadaddr=0x02000000          \0"

#undef CONFIG_USE_IRQ                  /* no support for IRQs */
#undef CONFIG_SKIP_LOWLEVEL_INIT       /* Boot from flash when undef'ed */

#define CONFIG_MISC_INIT_R
#define BOARD_LATE_INIT

#define CONFIG_CMDLINE_TAG	1	/* enable passing of ATAGs */
#define CONFIG_INITRD_TAG	1
#define CONFIG_REVISION_TAG	0
#define CONFIG_ETHERNET_TAG	1
#define CONFIG_SETUP_MEMORY_TAGS	1
#define CONFIG_DECARM_TAG  1
#define CONFIG_SHOW_BOOT_PROGRESS	1 
#define CONFIG_CONIN_TAG 1

/*
 * Hardware drivers
 */

/*
 * NS16550 Configuration
 */

#define V_NS16550_CLK	(V_SCLK)

#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE	(4)
#define CFG_NS16550_CLK	V_NS16550_CLK
#define ASX0_BASE               0xE0400000
#define SER_BASE                ASX0_BASE + 0x10000
#define CNXT_UART1              (SER_BASE + 0x00000)
#define CNXT_UART2              (SER_BASE + 0x01000)
#define CNXT_UART3              (SER_BASE + 0x02000)
#define CFG_NS16550_COM1	CNXT_UART1
#define CFG_NS16550_COM2	CNXT_UART2
#define CFG_NS16550_COM3	CNXT_UART3

/* Ethernet Commands */
#define PHY_ADD            0x10
#define CONFIG_NET_MULTI   1 
#define CONFIG_MII         1
#define CONFIG_UDP_CHECKSUM 1


/************************************************************
 * IDE/ATA stuff
 ************************************************************/
#define CFG_IDE_MAXBUS		1   /* has only one IDE bus	*/

#define CFG_IDE_MAXDEVICE	(CFG_IDE_MAXBUS*2) /* max. 2 drives per IDE bus */

#define CFG_ISA_IO_BASE_ADDRESS 0xE8000000
#define CFG_ATA_BASE_ADDR	CFG_ISA_IO_BASE_ADDRESS /* base address */
#define CFG_ATA_IDE0_OFFSET	0x01F0		/* ide0 offste */
#define CFG_ATA_IDE1_OFFSET	0x0170		/* ide1 offset */
#define CFG_ATA_DATA_OFFSET	0		/* data reg offset	*/
#define CFG_ATA_REG_OFFSET 	0		/* reg offset */
#define CFG_ATA_ALT_OFFSET	0x200		/* alternate register offset */

#undef	CONFIG_IDE_8xx_DIRECT      /* no pcmcia interface required */
#undef	CONFIG_IDE_LED	       /* no led for ide supported     */
#define CONFIG_IDE_RESET       /* reset for ide supported...	*/
#define CONFIG_IDE_RESET_ROUTINE /* with a special reset function */
#define CONFIG_SUPPORT_VFAT
/************************************************************
 * ATAPI support (experimental)
 ************************************************************/
//#define CONFIG_ATAPI			/* enable ATAPI Support */

/************************************************************
 * DISK Partition support
 ************************************************************/
//#define CONFIG_DOS_PARTITION
//#define CONFIG_MAC_PARTITION
//#define CONFIG_ISO_PARTITION /* Experimental */

/*
 * select serial console configuration
 */
#define CONFIG_SERIAL_MULTI 1
#define CONFIG_CONS_INDEX   1  // 3 MH609, 1 MH610
#define CONFIG_BAUDRATE	115200
#define CFG_BAUDRATE_TABLE	{9600, 19200, 38400, 57600, 115200}

/*-----------------------------------------------------------------------
 *  Command line configuration.
 */
#include <config_cmd_default.h>

/* BOOT COMMANDS */
#define CONFIG_AUTOSCRIPT  1
#define CONFIG_BOOTDELAY   1
#define CFG_LOAD_ADDR      0x02000000 /* default load address */
#undef CONFIG_BOOTARGS
/* #define CONFIG_BOOTARGS "console=ttyRI0 root=/dev/mtdblock3 rw rootfstype=jffs2 mem=512M" */
/* #define CONFIG_BOOTARGS "root=/dev/ram0 rw mem=512M " */ /* for ramdisk support */ 

#ifdef CONFIG_USE_NFC
#undef CONFIG_CMD_FLASH
#undef CONFIG_CMD_IMLS
#define CONFIG_BOOTCOMMAND "nfc read ${loadaddr} 0x100000 0x400000; bootm ${loadaddr}"
#elif defined(CONFIG_USE_NOR)
#define CONFIG_CMD_JFFS2
#define CONFIG_BOOTCOMMAND "bootm ${kernel_rom_addr}"
#else
#error "Either CONFIG_USE_NFC or CONFIG_USE_NOR _MUST_ be defined !!!"
#endif

#define NAND_BOOTCOMMAND "nfc read ${loadaddr} 0x100000 0x400000; bootm ${loadaddr}"
#define CONFIG_BOOTCOMMAND "bootm ${kernel_rom_addr}"

#define CONFIG_CMD_DHCP
#define CONFIG_CMD_MII
#define CONFIG_CMD_PING

/*-----------------------------------------------------------------------
 * Size of malloc() pool
 */
#define CFG_MALLOC_LEN        (CONFIG_ENV_SIZE+SZ_128K)
/*#define CFG_GBL_DATA_SIZE   128 */   /* size in bytes reserved for initial data */
#define CFG_GBL_DATA_SIZE      512     /* size in bytes reserved for initial data */

/*-----------------------------------------------------------------------
 * Miscellaneous configurable options
 */
#define CFG_PROMPT_HUSH_PS2	"> "
#define CFG_HUSH_PARSER
#define V_PROMPT		         "NXP # "

#define USE_MEMCPY		1

#define CFG_LONGHELP		/* undef to save memory */
#define CFG_PROMPT		V_PROMPT
#define CFG_CBSIZE		512 /* Console I/O Buffer Size */
/* Print Buffer Size */
#define CFG_PBSIZE		(CFG_CBSIZE+sizeof(CFG_PROMPT)+16)
#define CFG_MAXARGS		16          /* max number of command args */
#define CFG_BARGSIZE		CFG_CBSIZE  /* Boot Argument Buffer Size */

#define CFG_MEMTEST_START	0x02000000
#define CFG_MEMTEST_END		(2*1024*1024)  /* 2 MB */


#undef   CFG_CLKS_IN_HZ		/* everything, incl board info, in Hz */
#define CFG_TIMERBASE		CNXT_GPT1
#define CFG_HZ_CLOCK		   CONFIG_SYS_CLK_FREQ
#define CFG_HZ		   	   1000

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	SZ_256K	/* regular stack */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	SZ_256K	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	SZ_4K	/* FIQ stack */
#endif

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS  2  /* defined the max meminfo entries */

/*-----------------------------------------------------------------------
 * Configuration Registers
 */
#define CONFIG_CM_REMAP       1  /* Remap ROM RAM to address space 0x000 */

/*-----------------------------------------------------------------------
 * Compression format
 */
#define CONFIG_LZMA

#endif	/* __CONFIG_H */
