/*
 * NS16550 Serial Port
 * originally from linux source (arch/ppc/boot/ns16550.h)
 * modified slightly to
 * have addresses as offsets from CFG_ISA_BASE
 * added a few more definitions
 * added prototypes for ns16550.c
 * reduced no of com ports to 2
 * modifications (c) Rob Taylor, Flying Pig Systems. 2000.
 *
 * added support for port on 64-bit bus
 * by Richard Danter (richard.danter@windriver.com), (C) 2005 Wind River Systems
 */

#if defined(CONFIG_IRD)

#if (CFG_NS16550_REG_SIZE == 4)

struct NS16550 {
	unsigned char rbr;		/* 0 r  */
	int pad1:24;
	unsigned char ier;		/* 1 rw */
	int pad2:24;
	unsigned char fcr;		/* 2 w  */
	int pad3:24;
	unsigned char lcr;		/* 3 rw */
	int pad5:24;
	int pad4;
	unsigned char lsr;
	int pad41:24;

	/* Some registers not used
	 *      iir, irvl , irdc , txsta , rxsta 
	 * resister details are in datasheet
	 * 
 	 * Appropriately padding is added 
	 */
	int pad6[6];

	unsigned char exp;
	int pad8:24;
	unsigned char mcr;		/* 4 rw */
	int pad9:24;
	unsigned char msr;		/* 5 r  */
	int pad7:24;
} __attribute__ ((packed));

#else
#error "CFG_NS16550_REG_SIZE should be defined 4 in case of CONFIG_NS24XXX"
#endif

#define thr rbr
#define dll rbr
#define dlm ier

typedef volatile struct NS16550 *NS16550_t;

#define FCR_FIFO_EN_128 0x01            /* Fifo enable - enabled by default */
#define FCR_RXSR        0x02            /* Receiver soft reset - clear receive buffer */
#define FCR_TXSR        0x04            /* Transmitter soft reset - clear transmit buffer */

#define FCR_CNXT        0x31            /* TX & RX DMA setting from datasheet + 128 byte fifo */

#define MCR_DTR         0x01
#define MCR_RTS         0x02

/* Are these present in our UART ? */
#if 0
#define MCR_DMA_EN      0x04
#define MCR_TX_DFR      0x08
#endif

#define LCR_WLS_MSK     0x01            /* character length select mask */
#define LCR_WLS_7	0x00		/* 7 bit character length */
#define LCR_WLS_8       0x01            /* 8 bit character length */
#define LCR_LBM		0x02		/* Loopback mode */
#define LCR_STB         0x04            /* Number of stop Bits, off = 1, on = 1.5 or 2) */
#define LCR_PEN         0x08            /* Parity eneble */
#define LCR_EPS         0x10            /* Even Parity Select */
#define LCR_STKP        0x20            /* Stick Parity */
#define LCR_SBRK        0x40            /* Set Break */
#define LCR_BKSE        0x80            /* Bank select enable 
					 * - verify this register Baud Divisor Select in our datasheet 
					 */

#define LSR_DR          0x01            /* Data ready */
#define LSR_OE          0x02            /* Overrun */
#define LSR_PE          0x04            /* Parity error */
#define LSR_FE          0x08            /* Framing error */
#define LSR_BI          0x10            /* Break */
#define LSR_THRE        0x20            /* Xmit holding register empty */
#define LSR_TEMT        0x40            /* Xmitter empty */
#define LSR_ERR         0x80            /* Error */

#endif

void	NS16550_init   (NS16550_t com_port, int baud_divisor);
void	NS16550_putc   (NS16550_t com_port, char c);
char	NS16550_getc   (NS16550_t com_port);
int	NS16550_tstc   (NS16550_t com_port);
void	NS16550_reinit (NS16550_t com_port, int baud_divisor);

/* useful defaults for LCR */
#define LCR_8N1		0x01


