/*
 * (C) Copyright 2008 NXP
 *
 * (C) Copyright 2000-2005
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ********************************************************************
 * NOTE: This header file defines TLV image related definitions.
 ********************************************************************
 */

#if defined(CONFIG_TLV_IMAGE)
#include "basetype.h"
 
#define     TLV_TYPE_UNUSED         0xff
#define     TLV_TYPE_LENGTH         0x01   /* uncompressed length */
#define     TLV_TYPE_CRC            0x02
#define     TLV_TYPE_DEST_ADDR      0x03
#define     TLV_TYPE_END_ADDR       0x04   /* MFB:  Is this req'd?  DEST_ADDR + LENGTH == END? */
#define     TLV_TYPE_FLASH_ADDR     0x05
#define     TLV_TYPE_COPYRIGHT      0x06
#define     TLV_TYPE_IMAGE_TYPE     0x07
#define     TLV_TYPE_DESCRIP        0x08
#define     TLV_TYPE_BOOT_STRING    0x09

#define     TLV_TYPE_SHARED_RAM_LENGTH   0x11
#define     TLV_TYPE_RUNTIME_RAM_LENGTH  0x12

/* image data must be the last TLV in an image. */
#define     TLV_TYPE_IMAGE_DATA     0x10

#define     IMAGE_TYPE_UNUSED       0xFF
#define     IMAGE_TYPE_UNC_CLASSIC  0x01
#define     IMAGE_TYPE_COM_CLASSIC  0x81
#define     IMAGE_TYPE_UNC_KERNEL   0x02
#define     IMAGE_TYPE_COM_KERNEL   0x82
#define     IMAGE_TYPE_UNC_INITRD   0x03
#define     IMAGE_TYPE_COM_INITRD   0x83

#define     IMAGE_TYPE_UNC_SLAVE    0x04
#define     IMAGE_TYPE_COM_SLAVE    0x84

#define     IMAGE_TYPE_CM_BRANDINGINFO 0x05
#define     IMAGE_TYPE_CM_IMAGECODE    0x06

/* 0x32545843 */
#define     TLV_MAGIC_NUMBER_INT    ( (int)'2' << 24 | \
                                      (int)'T' << 16 | \
                                      (int)'X' <<  8 | \
                                      (int)'C' << 0       )

#define     TLV_MAGIC_NUMBER        "CXT2"

#ifndef __ASM_ARM_TYPES_H
typedef     unsigned char    u8;
typedef     unsigned short   u16;
typedef     unsigned long    u32;
#endif

/* beware of portability issues here!  this structure needs to be
 * 1-byte packed!!!  The lnxhdr32 utility has been written such
 * that this structure compiles correctly (i.e., pragma'd) and 
 * generates proper data when compiled with MSVC60.  However, 
 * when reading this data, it should be done a byte at a time 
 * and reassembled "by hand" to ensure ARM toolkit compatibility.
 */
typedef struct _tlv_entry_ {
    u32  tlv_type;
    u32  tlv_length;
    void *data; /* sometimes this will be a pointer, other times it will
                 * simply be the data since a pointer is 32 bits == sizeof(int)
                 */  
} tlv_entry, *ptlv_entry;

#define IMAGE_FORMAT_TLV	0x03	/* NXP TLV based format */

/* TLV block should be on 32k/64k boundary, but the code which searchs
   for TLV header assuming 16K. It is slower, but it works if the app
   decides to put the TLV header in 16K boundary
 */
#define TLV_BLOCK_MIN_SIZE      (0x10000>>2) /* 16K */

typedef struct
{
   u_int32 uImageType;
   void    *pSrcAddr;
   void    *pDstAddr;
   u_int32 uImageLen;
   bool    bCompressed;
   u_int32 uSharedRAMSize;
   u_int32 uRunTimeRAMSize;
   u_int32 uSharedRAMStart;
} TLV_IMAGE_INFO;

typedef enum
{
   CRC_CHECKING = 0,
   DECOMPRESSING,
   IMAGE_LOADING,
   TLV_IMAGE_PROCESS_PHASE_LAST = IMAGE_LOADING
} TLV_IMAGE_PROCESS_PHASE;

bool find_tlv_image(u_int32 uImageType, u_int32 **ppTLVEntry); 
bool find_and_load_tlv_image(u_int32 uImageType, u_int32 **ppuImageEntry);
bool get_tlv_image_info(u_int32 *puTLVAddr, TLV_IMAGE_INFO *pImageInfo); 
bool get_tlv_image_type( u_int32 *puTLVAddr, u_int32 *puImageType );
void list_tlv_images(void);


#endif
