/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <zlib.h>
#include <asm/byteorder.h>

/* defined in board/nxp/ird/ird.c */
extern u32 ATAG_ADDRESS; 
/* defined in board/nxp/common/decarm.c */
extern struct tag_decarm decarm_tag;

// AIX_3RDPARTY_CODE_CHANGED: multiple: look for CONIN_TAG...

DECLARE_GLOBAL_DATA_PTR;

#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_ETHERNET_TAG) || \
    defined (CONFIG_DECARM_TAG) || \
    defined (CONFIG_CONIN_TAG) || \
    defined (CONFIG_REVISION_TAG) || \
    defined (CONFIG_VFD) || \
    defined (CONFIG_LCD)
/* static*/ void setup_start_tag (bd_t *bd);

# ifdef CONFIG_SETUP_MEMORY_TAGS
/* static */void setup_memory_tags (bd_t *bd);
# endif
/* static */ void setup_commandline_tag (bd_t *bd, char *commandline);

# ifdef CONFIG_INITRD_TAG
static void setup_initrd_tag (bd_t *bd, ulong initrd_start,
			      ulong initrd_end);
# endif

#ifdef CONFIG_ETHERNET_TAG
/*static*/void setup_ethernet_tags(bd_t *bd);
#endif 

#ifdef CONFIG_DECARM_TAG
/*static*/void setup_decarm_tag (bd_t *bd, struct tag_decarm *decarm );
#endif 

#ifdef CONFIG_CONIN_TAG
/*static*/void setup_conin_tag (bd_t *bd);
#endif 

void setup_end_tag (bd_t *bd);

# if defined (CONFIG_VFD) || defined (CONFIG_LCD)
static void setup_videolfb_tag (gd_t *gd);
# endif

static struct tag *params;
#endif /* CONFIG_SETUP_MEMORY_TAGS || CONFIG_CMDLINE_TAG || CONFIG_INITRD_TAG */

static void print_ataglist ( u32 *puTagStart );
int do_bootm_linux(int flag, int argc, char *argv[], bootm_headers_t *images)
{
	bd_t	*bd = gd->bd;
	char	*s;
	int	machid = bd->bi_arch_number;
	void	(*theKernel)(int zero, int arch, uint params);
   void (*theMaster)(uint pAtaglistAddr, uint pEntry);
	int	ret;

#ifdef CONFIG_CMDLINE_TAG
	char *commandline = getenv ("bootargs");
#endif

   /* adjust the ataglist address for linux kernel boot */
   if( images->legacy_hdr_os_copy.ih_type == IH_TYPE_KERNEL )
   {
      /* kernel image, relocate the ataglist address */
      /* ep is the kernel entry address, it should always be
       * at 0xXXXX8000, set the ATAG_ADDRESS to be below the
         kernel address on 1M boundary */
      ATAG_ADDRESS =  (images->ep & ~(0x100000 -1));
      if( ATAG_ADDRESS == images->ep )
      {
         /* Kernel Entry should always at 0x8000 boundary, for some
          * reason this is not the case. we shall store the atag list
            1M below. */
         printf( " Kernel Entry is at 1M boundary already, store atag list 1M blow the kernel\n" );
         ATAG_ADDRESS -= 0x100000;
      }
      printf( "TAG_ADDRESS is at 0x%08x before the kernel 0x%08x\n", (unsigned int)ATAG_ADDRESS, (unsigned int)images->ep );

      bd->bi_boot_params = ATAG_ADDRESS;
   }

	theKernel = (void (*)(int, int, uint))images->ep;

	s = getenv ("machid");
	if (s) {
		machid = simple_strtoul (s, NULL, 16);
		printf ("Using machid 0x%x from environment\n", machid);
	}

	show_boot_progress (15);

	debug ("## Transferring control to Linux (at address %08lx) ...\n",
	       (ulong) theKernel);

#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_ETHERNET_TAG) || \
    defined (CONFIG_DECARM_TAG) || \
    defined (CONFIG_CONIN_TAG) || \
    defined (CONFIG_REVISION_TAG) || \
    defined (CONFIG_LCD) || \
    defined (CONFIG_VFD)
	setup_start_tag (bd);
#ifdef CONFIG_SERIAL_TAG
	setup_serial_tag (&params);
#endif
#ifdef CONFIG_REVISION_TAG
	setup_revision_tag (&params);
#endif
#ifdef CONFIG_SETUP_MEMORY_TAGS
	setup_memory_tags (bd);
#endif
#ifdef CONFIG_CMDLINE_TAG
	setup_commandline_tag (bd, commandline);
#endif
#ifdef CONFIG_INITRD_TAG
	if (images->rd_start && images->rd_end)
		setup_initrd_tag (bd, images->rd_start, images->rd_end);
#endif
#ifdef CONFIG_ETHERNET_TAG
	setup_ethernet_tags (bd);
#endif

#ifdef CONFIG_DECARM_TAG
   if( decarm_tag.code_size && decarm_tag.code_start )
   {
      setup_decarm_tag (bd, &decarm_tag );
   }
#endif

#ifdef CONFIG_CONIN_TAG
   setup_conin_tag (bd);
#endif

#if defined (CONFIG_VFD) || defined (CONFIG_LCD)
	setup_videolfb_tag ((gd_t *) gd);
#endif
	setup_end_tag (bd);
#endif

   print_ataglist( (u32 *)bd->bi_boot_params );


	/* we assume that the kernel is in place */
	printf ("\nStarting kernel ...\n\n");

#ifdef CONFIG_USB_DEVICE
	{
		extern void udc_disconnect (void);
		udc_disconnect ();
	}
#endif

	cleanup_before_linux ();
#if defined(CONFIG_TLV_IMAGE)
   if( images->legacy_hdr_os_copy.ih_type == IH_TYPE_KERNEL )
   {
	   printf ("\nStarting Kernel ...at address 0x%08x\n\n", (unsigned int)images->ep);
	   theKernel (0, machid, bd->bi_boot_params);
      /* does not return */
      return 1;
   }
   else
   {
      theMaster = (void (*)(uint, uint ))images->ep;
      printf ("\nStarting RTOS image ...at address 0x%08x\n\n", (unsigned int)images->ep);
      theMaster(bd->bi_boot_params, images->ep); 
   }
#else
	theKernel (0, machid, bd->bi_boot_params);
#endif

}


#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
    defined (CONFIG_CMDLINE_TAG) || \
    defined (CONFIG_INITRD_TAG) || \
    defined (CONFIG_SERIAL_TAG) || \
    defined (CONFIG_ETHERNET_TAG) || \
    defined (CONFIG_DECARM_TAG) || \
    defined (CONFIG_CONIN_TAG) || \
    defined (CONFIG_REVISION_TAG) || \
    defined (CONFIG_LCD) || \
    defined (CONFIG_VFD)
/*static*/ void setup_start_tag (bd_t *bd)
{
	params = (struct tag *) bd->bi_boot_params;

	params->hdr.tag = ATAG_CORE;
	params->hdr.size = tag_size (tag_core);

	params->u.core.flags = 0;
	params->u.core.pagesize = 0;
	params->u.core.rootdev = 0;

	params = tag_next (params);
}


#ifdef CONFIG_SETUP_MEMORY_TAGS
/*static*/ void setup_memory_tags (bd_t *bd)
{
	int i;

	for (i = 0; i < CONFIG_NR_DRAM_BANKS; i++) {
      /* only populate an entry with non-zero size */
      if( bd->bi_dram[i].size != 0 )
      {
	      params->hdr.tag = ATAG_MEM;
	      params->hdr.size = tag_size (tag_mem32);
       
         params->u.mem.start = bd->bi_dram[i].start;
         params->u.mem.size = bd->bi_dram[i].size;
       
	      params = tag_next (params);
      }
	}
}
#endif /* CONFIG_SETUP_MEMORY_TAGS */


/*static */void setup_commandline_tag (bd_t *bd, char *commandline)
{
	char *p;

	if (!commandline)
		return;

	/* eat leading white space */
	for (p = commandline; *p == ' '; p++);

	/* skip non-existent command lines so the kernel will still
	 * use its default command line.
	 */
	if (*p == '\0')
		return;

	params->hdr.tag = ATAG_CMDLINE;
	params->hdr.size =
		(sizeof (struct tag_header) + strlen (p) + 1 + 4) >> 2;

	strcpy (params->u.cmdline.cmdline, p);

	params = tag_next (params);
}


#ifdef CONFIG_INITRD_TAG
static void setup_initrd_tag (bd_t *bd, ulong initrd_start, ulong initrd_end)
{
	/* an ATAG_INITRD node tells the kernel where the compressed
	 * ramdisk can be found. ATAG_RDIMG is a better name, actually.
	 */
	params->hdr.tag = ATAG_INITRD2;
	params->hdr.size = tag_size (tag_initrd);

	params->u.initrd.start = initrd_start;
	params->u.initrd.size = initrd_end - initrd_start;

	params = tag_next (params);
}
#endif /* CONFIG_INITRD_TAG */


#if defined (CONFIG_VFD) || defined (CONFIG_LCD)
extern ulong calc_fbsize (void);
static void setup_videolfb_tag (gd_t *gd)
{
	/* An ATAG_VIDEOLFB node tells the kernel where and how large
	 * the framebuffer for video was allocated (among other things).
	 * Note that a _physical_ address is passed !
	 *
	 * We only use it to pass the address and size, the other entries
	 * in the tag_videolfb are not of interest.
	 */
	params->hdr.tag = ATAG_VIDEOLFB;
	params->hdr.size = tag_size (tag_videolfb);

	params->u.videolfb.lfb_base = (u32) gd->fb_base;
	/* Fb size is calculated according to parameters for our panel
	 */
	params->u.videolfb.lfb_size = calc_fbsize();

	params = tag_next (params);
}
#endif /* CONFIG_VFD || CONFIG_LCD */

#ifdef CONFIG_SERIAL_TAG
void setup_serial_tag (struct tag **tmp)
{
	struct tag *params = *tmp;
	struct tag_serialnr serialnr;
	void get_board_serial(struct tag_serialnr *serialnr);

	get_board_serial(&serialnr);
	params->hdr.tag = ATAG_SERIAL;
	params->hdr.size = tag_size (tag_serialnr);
	params->u.serialnr.low = serialnr.low;
	params->u.serialnr.high= serialnr.high;
	params = tag_next (params);
	*tmp = params;
}
#endif

#ifdef CONFIG_REVISION_TAG
void setup_revision_tag(struct tag **in_params)
{
	u32 rev = 0;
	u32 get_board_rev(void);

	rev = get_board_rev();
	params->hdr.tag = ATAG_REVISION;
	params->hdr.size = tag_size (tag_revision);
	params->u.revision.rev = rev;
	params = tag_next (params);
}
#endif  /* CONFIG_REVISION_TAG */

#ifdef CONFIG_ETHERNET_TAG
/*static*/ void setup_ethernet_tags(bd_t *bd)
{
   int i;

   params->hdr.tag = ATAG_MAC;
   params->hdr.size = ((sizeof(struct tag_header) + ((sizeof(struct tag_mac) + 3) & ~3)) >> 2);
   for (i = 0; i < 6; i++) {
   	params->u.mac.mac_address[i] = bd->bi_enetaddr[i];
   }

	params = tag_next(params);
}
#endif

#ifdef CONFIG_DECARM_TAG
/*static*/ void setup_decarm_tag (bd_t *bd, struct tag_decarm *decarm)
{
	/* an ATAG_DECARM node tells the kernel the load address of decarm firmware, 
    * the size of the firmware, the shared RAM location and size.
	 */
	params->hdr.tag = ATAG_DECARM;
	params->hdr.size = tag_size (tag_decarm);

	params->u.decarm.code_size  = decarm->code_size;
   params->u.decarm.code_start = decarm->code_start;
	params->u.decarm.sharedram_size  = decarm->sharedram_size;
   params->u.decarm.sharedram_start = decarm->sharedram_start;

	params = tag_next (params);
}
#endif /* CONFIG_DECARM_TAG */

#ifdef CONFIG_CONIN_TAG

extern int NS16550_input_disabled; // ns16550.c

/*static*/ void setup_conin_tag (bd_t *bd)
{
	/* an ATAG_DECARM node tells the kernel the load address of decarm firmware, 
    * the size of the firmware, the shared RAM location and size.
	 */
	params->hdr.tag = ATAG_CONIN;
	params->hdr.size = tag_size (tag_conin);

	params->u.conin.enabled  = !NS16550_input_disabled;

	params = tag_next (params);
}
#endif /* CONFIG_CONIN_TAG */

/*static*/ void setup_end_tag (bd_t *bd)
{
	params->hdr.tag = ATAG_NONE;
	params->hdr.size = 0;
}

#endif /* CONFIG_SETUP_MEMORY_TAGS || CONFIG_CMDLINE_TAG || CONFIG_INITRD_TAG */

static char* map_atag_type_to_str( u32 type )
{
   switch (type)
   {
      case ATAG_NONE:      return "NONE";
      case ATAG_CORE:      return "CORE";
      case ATAG_MEM:       return "MEMINFO";
      case ATAG_VIDEOTEXT: return "VIDEOTEX";
      case ATAG_RAMDISK:   return "RAMDISK";
      case ATAG_SERIAL:    return "SERIAL";
      case ATAG_REVISION:  return "REVISION";
      case ATAG_VIDEOLFB:  return "VIDEOLFB";
      case ATAG_CMDLINE:   return "CMDLINE";
      case ATAG_MAC:       return "MAC";
      case ATAG_DECARM:    return "DECARM";
      case ATAG_CONIN:     return "CONIN";
      case ATAG_INITRD2:   return "INITRD2";
      default:             return "UNKNOWN TYPE";
   }
} /* map_atag_type_to_str */

/*  
 * This function prints out the ATAG list 
 */
static void print_ataglist ( u32 *puTagStart )
{
   u32 uSize, uLoop, uCount, type;

   printf("Printing ATAG list at address 0x%08x\n", (u32)puTagStart);

   while ( 1 )
   {
      uSize = *(puTagStart);
      if( uSize == 0 )
      {
         /* the end of atag */
         printf(" ATAG_TYPE: END\n");
         break;
      }
      type = *(puTagStart+1);
      printf(" ATAG_TYPE: 0x%08x - %s\n", type, map_atag_type_to_str(type));
      printf(" Size     : 0x%04x\n", uSize );
      puTagStart+=2;
      printf(" Data     : ");

      uCount = 0;
      if( type == ATAG_CMDLINE )
      {
         u32 cmdlinelen = strlen( (char*)puTagStart);
         /* CFG_PBSIZE is the maximum size of string that printf can handle, check 
          * to see if the string is too big. 
          */
         if( cmdlinelen > CFG_PBSIZE )
         {
            printf("CMDLINE string length %dB is greater than what printf to handle %dB, skip printing.\n",
                   cmdlinelen, CFG_PBSIZE);
         }
         else
         {
            printf("%s\n", (char*)puTagStart);
         }
         puTagStart+=(uSize - 2);
      }
      else
      {
         for ( uLoop = 0; uLoop < (uSize - 2); uLoop++)
         {
            printf(" 0x%08x,", *puTagStart++);
            uCount++;
            if( (uCount % 6) == 0 )
            {
               printf("\n           ");
            }
         }
      }
      printf("\n");
      /* go to the next atag */
   }

   return;

} /* print_ataglist */
